


from Anapass.TModule import *
from Anapass import FlashMemory


#T5 Device에 연결한다. 
device = TDevice(TDevice.Type.T5)
isOk = device.Connect()
if isOk != True :
    raise ConnectionRefusedError("Fail to connect to TDevice(%s)"%device.GetName())

#Flash Memory Instacne 생성
#  TChip.Type.ANA6705 :  Anapass/ANA6705/ANA6705_FlashMipiToSp.py 에  코드 구현됨 
#  TChip.Type.ANA6706 :  Anapass/ANA6706/ANA6706_FlashMipiToSp.py 에  코드 구현됨 
chipType = TChip.Type.ANA6705
#chipType = TChip.Type.ANA6706
flash = FlashMemory.CreateInstance(device, chipType, FlashMemory.Type.MipiToSpi)


writeMemAddr = 0x1000
writeByteSize = 512 
writeData=[x&0xFF for x in range(writeByteSize)]  #1 sector 만큼 읽는다.

print("DataWrite.... ", end= ' ')
isOk = flash.Write(writeMemAddr, writeByteSize, writeData )
assert isOk
print("OK")
Util.Print.ListHex(writeMemAddr, writeData)
print()

print("DataRead.... ", end= ' ')
readData=[0 for _ in range(writeByteSize)]  
isOk = flash.Read(writeMemAddr, writeByteSize, readData )
assert isOk
print("OK")
Util.Print.ListHex(writeMemAddr, readData)
print()

print("Data Verify ... ", end= ' ')
isVerify=True
for idx in range(writeByteSize) :
    if writeData[idx] != readData[idx] :
        isVerify = False
        break

if isVerify : 
    print("Success")
else :
    print("Fail")
    print("Fail")

#T5 Device 접속 종료
device.Disonnect()






from Anapass.TModule import *
from Anapass.AppShell import AppClass
from Anapass.AppShell import CommandTFlash


def TestFunc1(app, strArg) :
   print("TestFunc1= ", strArg)

def TestFunc2(app, strArg) :
   print("TestFunc2= ", strArg)

app = AppClass.AppGlobal()    
app.SetChipType(TChip.Type.ANA6705)
#app.SetChipType(TChip.Type.ANA6706)

PROMPT="TEDTools>> "

print("----------------------------------------------------")
print("App: Python MyShell")
print("----------------------------------------------------")

shellCmd = AppClass.ShellCommand()
#shellCmd.Add("Test1", TestFunc1, "TestFunc1")
#shellCmd.Add("Test2", TestFunc1, "TestFunc2")
shellCmd.Add("Flash", CommandTFlash.Main, "Flash Memory Read/Write")

for shellItem in shellCmd :
    print(shellItem)

while True : 
    print(PROMPT, end='')
    str = input()
    #print("Input=[" + str + "]")
    strlist = str.split(' ')
    
    if( (strlist[0] == 'exit') and (len(strlist)==1) ) :
        break

    cmd=""
    argIdx = 0
    listArg=list()
    for s in strlist :
        if len(s) != 0 :
            if argIdx == 0 :
                cmd = s.lower()
                argIdx += 1
            else :
                listArg.append(s.lower())
    tupleArg = tuple(listArg)
    if len(cmd) > 0 :
        if cmd == 'help' or cmd == '?' : 
            shellCmd.PrintHelp()
        else :
            isCmdRun = False
            for shellItem in shellCmd :
                if shellItem.Run(app, cmd, tupleArg) == True :
                    isCmdRun = True
                    break

            if isCmdRun == False :
                print("Can't find command")





class WorkAgent :
    def __init__(this):
        this.__IsBreak = False
        this.__IsStart = False
        this.__IsStop = False
        this.__MaxValue = 0
        this.__MinValue = 0
        this.__CurValue = 0

    def Start(this) :
        this.__IsStart = True

    def Stop(this) :
        this.__IsStop = True

    def IsStop(this) :
        return this.__IsStop;

    def SetMaxValue(this, v) :
        this.__MaxValue = v

    def GetMaxValue(this) :
        return this.__MaxValue

    def SetMinValue(this, v) :
        this.__MinValue = v

    def SetCurValue(this, v) :
        this.__CurValue = v

    def GetCurValue(this) :
        return this.__CurValue

    def IncCurValue(this, incV) :
        this.__CurValue += incV


class Print :
    def ListHex(startMemAddr, list) :
        lineByte = 16
        i = 0
        strline = ""
        for v in list :
            strline += "0x%02X "%v
            if (i+1)%lineByte == 0 :
                j = ((int)(i/lineByte)*lineByte) + startMemAddr
                print("0x%08X : "%  j, strline)
                strline=""
            i += 1
        if len(strline) > 0 :
            j = ((int)(i/lineByte)*lineByte) + startMemAddr
            print("0x%08X : "% j , strline)


def ListCopy(list1, startIdx1, list2, startIdx2, copyCount) :
    for i in range(copyCount) :
        list1[startIdx1 + i] = list2[startIdx2 + i]

def ListCopyExam() :
    list1 = [1, 2, 3, 4, 5, 6]
    list2 = [33, 44, 55]
    print("list1= ", list1)
    print("list2= ", list2)
    Util.ListCopy(list1, 2, list2, 0, 3)
    print("list1= ", list1)
    print("list2= ", list2)


class List :
    def ToHexString(list) :
        strList = ""
        for v in list :
            strList += "0x%02X "%v
        return strList            




import platform
import enum
from Anapass.TDeviceBase import *

if platform.system()=="Windows" : 
    from Anapass.TDeviceWin import *
else :
    from Anapass.TDeviceAndroid import *

class TDevice :

    class Type(enum.IntEnum) : 
        T5 = 0
        T5PacketAnalysis=1
        T4 = 2
        TESys=3
    
    def __init__(this, deviceType): 

        if platform.system()=="Windows" : 
            print("os is windows")
            this.__TDevice = TDeviceWin(deviceType.value)
        else :
            this.__TDevice = TDeviceAndroid(deviceType.value)

        this.__DeviceType = deviceType

        #Staticts
        this.__AgingMeasureAdcCount = 0


    def __getattr__(this, attrName) :
        
        if attrName == 'Message' :
            return this.DebugMessage
        
        elif attrName == 'PTRN_SET' :
            return this.PatternSetCommand
        
        elif attrName == 'WREG0_39' :
            return this.DD_DSIM_MipiWriteReg39
        
        elif attrName == 'WREG0_15' :
            return this.DD_DSIM_MipiWriteReg15
        
        elif attrName == 'WREG0_05' :
            return this.DD_DSIM_MipiWriteReg05

        elif attrName == 'WREG0_07' :
            return this.DD_DSIM_MipiWriteReg07
        
        elif attrName == 'RREG1Byte' :
            return this.DD_DSIM_MipiReadReg1Byte
        
        elif attrName == 'RREG' :
            return this.DD_DSIM_MipiReadReg
        
        elif attrName == 'WREG1Byte' :
            return this.DD_DSIM_MipiWriteReg1Byte
        
        elif attrName == 'WREG' :
            return this.DD_DSIM_MipiWriteReg
        
        elif attrName == 'DelayMS' :
            return this.SysDelay

        else :
            raise AttributeError(attrName)


    def GetName(this) :
        return this.__DeviceType.name

    def SysSetServerIPAddr(this, serverIPAddr) : 
        return this.__TDevice.SysSetServerIPAddr(serverIPAddr);
    
    def Connect(this) : 
        return this.__TDevice.Connect()

    def Disonnect(this) : 
        return this.__TDevice.Disonnect()

    def SysSetBoardID(this, boardID) : 
        return this.__TDevice.SysSetBoardID(boardID)
    
    def SetTcLocalSave(this, boardID, bFlag) : 
        return this.__TDevice.SetTcLocalSave(boardID, bFlag)
        
    def SysGetDutIndexAllDeviceValue(this) : 
        return this.__TDevice.SysGetDutIndexAllDeviceValue()

    def SysGetDutCount(this) : 
        return this.__TDevice.SysGetDutCount()

    def SysGetTickCount64(this) : 
        return this.__TDevice.SysGetTickCount64()

    def SysGetCurUtcTime(this) : 
        return this.__TDevice.SysGetCurUtcTime()

    def SysGetUtcTimeKST(this, year, month, day, hour, min, sec) : 
        return this.__TDevice.SysGetUtcTimeKST(year, month, day, hour, min, sec)

    def SysGetErrFlag(this) : 
        return this.__TDevice.SysGetErrFlag()

    def SysMipiLock(this) : 
        return this.__TDevice.SysMipiLock()

    def SysMipiUnlock(this) : 
        return this.__TDevice.SysMipiUnlock()

    def SysMipiIsLock(this) : 
        return this.__TDevice.SysMipiIsLock()

    def SendTxtCmd(this, cmd) : 
        return this.__TDevice.SendTxtCmd(cmd)

    def SendTxtCmdReadResp(this, cmd, maxRespByteSize) : 
        return this.__TDevice.SendTxtCmdReadResp(cmd, maxRespByteSize)

    def SendCtrlCmd(this, cmd) :  
        return this.__TDevice.SendCtrlCmd(cmd)

    def Reset(this) :
        return this.__TDevice.Reset()

    def Next(this) :
        return this.__TDevice.Next()

    def Back(this) :
        return this.__TDevice.Back()

    def ReadReg(this, regAddr, byteOffset, readCount, regValueList, regValueListStartIdx=0) : 
        
        dutIndex = 0
        regValueInt = 0
        regValueByteArray=bytes(readCount)
        ret = this.__TDevice.DD_DSIM_MipiReadReg(this.__DeviceHandle, regAddr, byteOffset, readCount, regValueByteArray)
        if ret==1 :
            for idx, regValueByte in enumerate(regValueByteArray) :
                regValueInt = regValueByte
                regValueInt &= 0xFF
                regValueList[idx + regValueListStartIdx] = regValueInt
            bflag = True
        else :
            bflag = False;
        return bflag

    def ReadReg1Byte(this, regAddr, byteOffset) : 
        return this.__TDevice.ReadReg1Byte(regAddr, byteOffset)

    def WriteReg(this, regAddr, byteOffset, writeCount, regValueList, writeDataStartIdx=0) : 
        return this.__TDevice.WriteReg(regAddr, byteOffset, writeCount, regValueList, writeDataStartIdx)

    def WriteReg1Byte(this, regAddr, byteOffset, regValue) : 
        return this.__TDevice.WriteReg1Byte(regAddr, byteOffset, regValue)
        
    def WriteCtrlReg(this, regAddr) : 
        return this.__TDevice.WriteCtrlReg(regAddr)

    #COMM_API Bool TedDD_DSIM_MipiReadReg(int dutIdx, int addr, int byteOffset, int readCount, unsigned char* buf, int bufMaxByteSize);
    def DD_DSIM_MipiReadReg(this, dutIdx, regAddr, byteOffset, readCount) : 
        return this.__TDevice.DD_DSIM_MipiReadReg(dutIdx, regAddr, byteOffset, readCount)

    def DD_DSIM_MipiReadReg1Byte(this, dutIdx, regAddr, byteOffset) : 
        return this.__TDevice.DD_DSIM_MipiReadReg1Byte(dutIdx, regAddr, byteOffset)

    #COMM_API Bool TedDD_DSIM_MipiReadReg(int dutIdx, int addr, int byteOffset, int readCount, unsigned char* buf, int bufMaxByteSize);
    def DD_DSIM_MipiWriteReg(this, dutIdx, regAddr, byteOffset, regValueList) : 
        return this.__TDevice.DD_DSIM_MipiWriteReg(dutIdx, regAddr, byteOffset, regValueList)

    #COMM_API Bool TedDD_DSIM_MipiWriteReg1Byte(int dutIdx, int addr, int byteOffset, unsigned char data);
    def DD_DSIM_MipiWriteReg1Byte(this, dutIdx, regAddr, byteOffset, regValue) : 
        return this.__TDevice.DD_DSIM_MipiWriteReg1Byte(dutIdx, regAddr, byteOffset, regValue)

    #WREG0=0x39, [Addr], [regVal0], [regVal1].....
    def DD_DSIM_MipiWriteReg39(this, dutIdx, regAddr, regValueList) : 
        return this.__TDevice.DD_DSIM_MipiWriteReg39(dutIdx, regAddr, regValueList)

    #WREG0=0x15, [Addr], [regVal]
    def DD_DSIM_MipiWriteReg15(this, dutIdx, regAddr, regValue) : 
        return this.__TDevice.DD_DSIM_MipiWriteReg15(dutIdx, regAddr, regValue)

    #WREG0=0x05, [Addr]
    def DD_DSIM_MipiWriteReg05(this, dutIdx, regAddr) : 
        return this.__TDevice.DD_DSIM_MipiWriteReg05(dutIdx, regAddr)

    #WREG0=0x07, [value]   :   Compressd Mode Command
    def DD_DSIM_MipiWriteReg07(this, dutIdx, value) : 
        return this.__TDevice.DD_DSIM_MipiWriteReg07(dutIdx, value)
        
    def CatchPower(this, powerInfo) : 
        return this.__TDevice.CatchPower(powerInfo)

    def PatternConnect(this) : 
        return this.__TDevice.PatternConnect()

    def PatternDisconnect(this) : 
        return this.__TDevice.PatternDisconnect()

    def PatternIsConnect(this) : 
        return this.__TDevice.PatternIsConnect()

    def PatternSetCommand(this, ptrnCmd) : 
        return this.__TDevice.PatternSetCommand(ptrnCmd)

    def PatternPaint(this, r, g, b, a) : 
        return this.__TDevice.PatternPaint(r, g, b, a)

    def PatternUpdateScreen(this) : 
        return this.__TDevice.PatternUpdateScreen()

    def PatternScreenVerify(this, r, g, b) : 
        return this.__TDevice.PatternScreenVerify(r, g, b)

    def PatternDrawImage(this, imgFileName) : 
        return this.__TDevice.PatternDrawImage(imgFileName)

    def ANA670X_GetChipIDCount(this) : 
        return this.__TDevice.ANA670X_GetChipIDCount()

    def ANA670X_GetChipID(this, dutIdx) : 
        return this.__TDevice.ANA670X_GetChipID(dutIdx)

    def ANA670X_GetProductRevisionBytesCount(this) : 
        return this.__TDevice.ANA670X_GetProductRevisionBytesCount()

    def ANA670X_GetProductRevisionBytes(this, dutIdx) : 
        return this.__TDevice.ANA670X_GetProductRevisionBytes(dutIdx)

    def ANA670X_SetFrameRate(this, dutIdx, fr) :
        return this.__TDevice.ANA670X_SetFrameRate(dutIdx, fr)

    def ANA670X_GetFrameRate(this, dutIdx) : 
        return this.__TDevice.ANA670X_GetFrameRate(dutIdx)

    def SysDelay(this, delay) : 
        return this.__TDevice.SysDelay(delay)
        
    def DebugMessage(this, msg) : 
        return this.__TDevice.DebugMessage(msg)
        
    def DebugFuncEnter(this, funcName) : 
        return this.__TDevice.DebugFuncEnter(funcName)
    
    def DebugFuncLeave(this, funcName) : 
        return this.__TDevice.DebugFuncLeave(funcName)
    
    def DD_FB_blank(this, dutIdx, value) : 
        return this.__TDevice.DD_FB_blank(dutIdx, value)

    def DD_DSIM_manual_ctrl(this, dutIdx, value) : 
        return this.__TDevice.DD_DSIM_manual_ctrl(dutIdx, value)

    def DD_DSIM_power_ctrl(this, dutIdx, value) : 
        return this.__TDevice.DD_DSIM_power_ctrl(dutIdx, value)

    def DD_DSIM_source_cal(this, dutIdx, value) : 
        return this.__TDevice.DD_DSIM_source_cal(dutIdx, value)

    def DD_DSIM_sleepin(this, dutIdx, value) : 
        return this.__TDevice.DD_DSIM_sleepin(dutIdx, value)
    
    def DD_DSIM_sleepout(this, dutIdx, value) : 
        return this.__TDevice.DD_DSIM_sleepout(dutIdx, value)
    
    def DD_DSIM_deep_standby(this, dutIdx, value) : 
        return this.__TDevice.DD_DSIM_deep_standby(dutIdx, value)
    
    def DD_DSIM_displayon(this, dutIdx, value) : 
        return this.__TDevice.DD_DSIM_displayon(dutIdx, value)
    
    def DD_DSIM_reset_ctrl(this, dutIdx, value) : 
        return this.__TDevice.DD_DSIM_reset_ctrl(dutIdx, value)


    #//VLIN1_ADC
    def Vlin1AdcSetSamples(this, value) : 
        return this.__TDevice.Vlin1AdcSetSamples(value)

    def Vlin1AdcSetInterval(this, value) : 
        return this.__TDevice.Vlin1AdcSetInterval(value)

    def Vlin1AdcSetChannelOn(this, value) : 
        return this.__TDevice.Vlin1AdcSetChannelOn(value)

    def Vlin1AdcSetChannelOff(this, value) : 
        return this.__TDevice.Vlin1AdcSetChannelOff(value)

    def Vlin1AdcSetMode(this, value) : 
        return this.__TDevice.Vlin1AdcSetMode(value)

    def Vlin1AdcGetVoltage(this, chIdx) : 
        return this.__TDevice.Vlin1AdcGetVoltage(chIdx)

    def Vlin1AdcGetVoltageVLIN1(this) : 
        return this.__TDevice.Vlin1AdcGetVoltageVLIN1()

    def Vlin1AdcGetVoltageVBAT(this) : 
        return this.__TDevice.Vlin1AdcGetVoltageVBAT()

    def Vlin1AdcGetVoltageELVDD(this) : 
        return this.__TDevice.Vlin1AdcGetVoltageELVDD()

    def Vlin1AdcGetCurrent(this, chIdx) : 
        return this.__TDevice.Vlin1AdcGetCurrent(chIdx)

    def Vlin1AdcGetCurrentVLIN1(this) : 
        return this.__TDevice.Vlin1AdcGetCurrentVLIN1()

    def Vlin1AdcGetCurrentVBAT(this) : 
        return this.__TDevice.Vlin1AdcGetCurrentVBAT()

    def Vlin1AdcGetCurrentELVDD(this) : 
        return this.__TDevice.Vlin1AdcGetCurrentELVDD()

    #//VCI_ADC
    def VciAdcSetSamples(this, value) : 
        return this.__TDevice.VciAdcSetSamples(value)

    def VciAdcSetInterval(this, value) : 
        return this.__TDevice.VciAdcSetInterval(value)

    def VciAdcSetChannelOn(this, value) : 
        return this.__TDevice.VciAdcSetChannelOn(value)

    def VciAdcSetChannelOff(this, value) : 
        return this.__TDevice.VciAdcSetChannelOff(value)

    def VciAdcSetMode(this, value) : 
        return this.__TDevice.VciAdcSetMode(value)
        
    def VciAdcGetVoltage(this, chIdx) : 
        return this.__TDevice.VciAdcGetVoltage(chIdx)

    def VciAdcGetVoltageVCI(this) : 
        return this.__TDevice.VciAdcGetVoltageVCI()

    def VciAdcGetVoltageVDDR(this) : 
        return this.__TDevice.VciAdcGetVoltageVDDR()

    def VciAdcGetVoltageVDDI(this) : 
        return this.__TDevice.VciAdcGetVoltageVDDI()

    def VciAdcGetCurrent(this, chIdx) : 
        return this.__TDevice.VciAdcGetCurrent(chIdx)

    def VciAdcGetCurrentVCI(this) : 
        return this.__TDevice.VciAdcGetCurrentVCI()

    def VciAdcGetCurrentVDDR(this) : 
        return this.__TDevice.VciAdcGetCurrentVDDR()

    def VciAdcGetCurrentVDDI(this) : 
        return this.__TDevice.VciAdcGetCurrentVDDI()

    #//SDOUT ADC
    def SoutAdcSetDevConfig(this, value) : 
        return this.__TDevice.SoutAdcSetDevConfig(value)

    def SoutAdcSetInConfig(this, chIdx, value) : 
        return this.__TDevice.SoutAdcSetInConfig(chIdx, value)

    def AdcSoutSetRBSel(this, dutIdx, value) : 
        return this.__TDevice.AdcSoutSetRBSel(dutIdx, value)

    def AdcSoutGetRBSel(this, dutIdx) : 
        return this.__TDevice.AdcSoutGetRBSel(dutIdx)

    def SoutAdcGetChannelCount(this) : 
        return this.__TDevice.SoutAdcGetChannelCount()

    def SoutAdcGetVoltage(this, chIdx) : 
        return this.__TDevice.SoutAdcGetVoltage(chIdx)

    def SoutAdcGetAllVoltage(this) : 
        return this.__TDevice.SoutAdcGetVoltage(chIdx)

    def LdoAdcSetInConfig(this, chIdx, value) : 
        return this.__TDevice.LdoAdcSetInConfig(chIdx, value)

    def LdoAdcGetChannelCount(this) : 
        return this.__TDevice.LdoAdcGetChannelCount()

    def LdoAdcGetVoltage(this, chIdx) : 
        return this.__TDevice.LdoAdcGetVoltage(chIdx)

    def LdoAdcGetAllVoltage(this) : 
        return this.__TDevice.LdoAdcGetAllVoltage()

    def RegAdcSetInConfig(this, chIdx, value) : 
        return this.__TDevice.RegAdcSetInConfig(chIdx, value)

    def RegAdcGetChannelCount(this) : 
        return this.__TDevice.RegAdcGetChannelCount()

    def RegAdcGetVoltage(this, chIdx) : 
        return this.__TDevice.RegAdcGetVoltage(chIdx)

    def RegAdcGetAllVoltage(this) : 
        return this.__TDevice.RegAdcGetAllVoltage()

    def AgingNotifyPyStart(this, pyFileName) : 
        return this.__TDevice.AgingNotifyPyStart(pyFileName)
    
    def AgingNotifyPyStop(this, pyFileName) : 
        return this.__TDevice.AgingNotifyPyStop(pyFileName)

    def AgingSetCurJobInfo(this, dutIdx, jobID, status, scIdx, scCount, desc) : 
        return this.__TDevice.AgingSetCurJobInfo(dutIdx, jobID, status, scIdx, scCount, desc)

    def AgingSetCurScInfo(this, dutIdx, scID, status, tcIdx, tcCount, desc) : 
        return this.__TDevice.AgingSetCurScInfo(dutIdx, scID, status, tcIdx, tcCount, desc)

    def AgingSetCurTcInfo(this, dutIdx, tcID, status, tcStepIdx, tcStepCount, desc) : 
        return this.__TDevice.AgingSetCurTcInfo(dutIdx, tcID, status, tcStepIdx, tcStepCount, desc)

    def AgingSetCurTcStepInfo(this, dutIdx, tcStepID, status, desc) : 
        return this.__TDevice.AgingSetCurTcStepInfo(dutIdx, tcStepID, status, desc)

    def AgingMeasureADC(this, dutIdx) : 
        this.__TDevice.AgingMeasureADC(dutIdx)
        #resList = this.__TDevice.AgingMeasureADC(dutIdx)
        #if(len(resList) > 0) :
        #    this.__AgingMeasureAdcCount+=1
        #return resList

    def GetAgingMeasureADCCount(this) :
        return this.__AgingMeasureAdcCount









from enum import Enum
import time
import platform
import sys
import enum
from abc import ABCMeta, abstractmethod
from Anapass import Util
from Anapass import Adc
import inspect

class TString :
    def __init__(this, str):
        this.__String = str

    def ToCTypeString(this) :
        return this.__String.encode('utf-8')

    #static method
    def ConvertToCTypeStrng(x) :
        return x.encode('utf-8')

    def ConvertCTypeStringToUnicode(x) :
        return x.decode('utf-8')

class TPower :

    class Type(enum.IntEnum) :
        VBAT1=0
        ELVSS=1
        VDD1=2
        VCI1=3
        VBAT2=4
        VDD2=5
        VCI2=6

    def __init__(this):
        this.No = 0
        this.Avail=[0 for _ in range(10)]
        this.Value1=[0 for _ in range(10)]
        this.Voltage=[0.0 for _ in range(10)]
        this.Current=[0.0 for _ in range(10)]
        this.Range1=[0.0 for _ in range(10)]
        this.Range2=[0.0 for _ in range(10)]


class TChip :
    class Type(enum.IntEnum) : 
        Common=0
        ANA6705=1
        ANA6706=2

class TBoard :
    class Type(enum.IntEnum) : 
        Common=0

#
# class TDevice
#
class TDeviceBase(metaclass=ABCMeta) :

    class Type(enum.IntEnum) : 
        T5 = 0
        T5PacketAnalysis=1
        T4 = 2
        TESys=3

    class ErrorString(enum.Enum) :
        GetResp="ErrorGetResp"
    
    def __init__(this, deviceTypeValue):
        #print(DisplayName +"TRY: create " + deviceType.name )
        this.__DeviceTypeValue = deviceTypeValue
        this.__SoutAdcMaxChCount=16
        this.__LdoAdcMaxChCount=16
        this.__RegAdcMaxChCount=16

    def __del__(this):
        #print("TDevice::~TDevice")
        TDeviceDestroy(this.__DeviceHandle)
    
    
    def TestVlin1Adc(this) :
        this.Vlin1AdcSetSamples(1111)
        this.Vlin1AdcSetInterval(22222)
        this.Vlin1AdcSetChannelOn(33333)
        this.Vlin1AdcSetChannelOff(44444)
        this.Vlin1AdcSetMode(55555)
        print("Volt(0) : %d "%this.Vlin1AdcGetVoltage(0))
        print("Volt-VLIN1 : %d "%this.Vlin1AdcGetVoltageVLIN1())
        print("Volt-VBAT : %d "%this.Vlin1AdcGetVoltageVBAT())
        print("Volt-ELVDD : %d "%this.Vlin1AdcGetVoltageELVDD())
        print("Curr(0) : %d "%this.Vlin1AdcGetCurrent(0))
        print("Curr-VLIN1 : %d "%this.Vlin1AdcGetCurrentVLIN1())
        print("Curr-VBAT : %d "%this.Vlin1AdcGetCurrentVBAT())
        print("Curr-ELVDD : %d "%this.Vlin1AdcGetCurrentELVDD())

    def TestVciAdc(this) :
        this.VciAdcSetSamples(111)
        this.VciAdcSetInterval(222)
        this.VciAdcSetChannelOn(333)
        this.VciAdcSetChannelOff(444)
        this.VciAdcSetMode(555)
        print("Volt(0) : %d "%this.VciAdcGetVoltage(0))
        print("Volt-VCI : %d "%this.VciAdcGetVoltageVCI())
        print("Volt-VDDR : %d "%this.VciAdcGetVoltageVDDR())
        print("Volt-VDDI : %d "%this.VciAdcGetVoltageVDDI())
        print("Curr(0) : %d "%this.VciAdcGetCurrent(0))
        print("Curr-VCI : %d "%this.VciAdcGetCurrentVCI())
        print("Curr-VDDR : %d "%this.VciAdcGetCurrentVDDR())
        print("Curr-VDDI : %d "%this.VciAdcGetCurrentVDDI())

    def TestSoutAdc(this) :
        this.SoutAdcSetDevConfig(77777)
        for chIdx in range(this.__SoutAdcMaxChCount) :
            this.SoutAdcSetInConfig(chIdx, 6600+chIdx)
        this.SoutAdcSetRBSel(1)
        for chIdx in range(this.__SoutAdcMaxChCount) :
            print("SoutAdc-Volt(%d) : %d "%(chIdx, this.SoutAdcGetVoltage(chIdx)))
        voltList=this.SoutAdcGetAllVoltage()
        for chIdx in range(this.__SoutAdcMaxChCount) :
            print("SoutAdc-VoltAll(%d) : %d "%(chIdx, voltList[chIdx]))
    
    def TestLdoAdc(this) :
        for chIdx in range(this.__LdoAdcMaxChCount) :
            this.LdoAdcSetInConfig(chIdx, 4500+chIdx)
        for chIdx in range(this.__LdoAdcMaxChCount) :
            print("LdoAdc-Volt(%d) : %d "%(chIdx, this.LdoAdcGetVoltage(chIdx)))
        voltList=this.LdoAdcGetAllVoltage()
        for chIdx in range(this.__LdoAdcMaxChCount) :
            print("LdoAdc-VoltAll(%d) : %d "%(chIdx, voltList[chIdx]))
    
    def TestRegAdc(this) :
        for chIdx in range(this.__RegAdcMaxChCount) :
            this.RegAdcSetInConfig(chIdx, 3400+chIdx)
        for chIdx in range(this.__RegAdcMaxChCount) :
            print("RegAdc-Volt(%d) : %d "%(chIdx, this.RegAdcGetVoltage(chIdx)))
        voltList=this.RegAdcGetAllVoltage()
        for chIdx in range(this.__RegAdcMaxChCount) :
            print("RegAdc-VoltAll(%d) : %d "%(chIdx, voltList[chIdx]))
    
    @abstractmethod            
    def SysSetServerIPAddr(this, serverIPAddr) : pass

    @abstractmethod            
    def SysSetBoardID(this, boardID) : pass

    @abstractmethod            
    def SetTcLocalSave(this, boardID, bFlag) : pass
        
    @abstractmethod            
    def SysGetDutIndexAllDeviceValue(this) : pass

    @abstractmethod            
    def SysGetDutCount(this) : pass

    @abstractmethod            
    def SysGetTickCount64(this) : pass

    @abstractmethod            
    def SysGetCurUtcTime(this) : pass

    @abstractmethod            
    def SysGetUtcTimeKST(this, year, month, day, hour, min, sec) : pass

    @abstractmethod            
    def SysGetErrFlag(this) : pass

    @abstractmethod            
    def SysMipiLock(this) : pass

    @abstractmethod            
    def SysMipiUnlock(this) : pass

    @abstractmethod            
    def SysMipiIsLock(this) : pass

    @abstractmethod            
    def Connect(this) : pass

    @abstractmethod            
    def Disonnect(this) : pass
        
    @abstractmethod            
    def SendTxtCmd(this, cmd) : pass

    @abstractmethod            
    def SendTxtCmdReadResp(this, cmd, maxRespByteSize) : pass

    @abstractmethod            
    def SendCtrlCmd(this, cmd) :  pass

    def Reset(this) :
        return this.SendCtrlCmd('RESET')

    def Next(this) :
        return this.SendCtrlCmd('NEXT')

    def Back(this) :
        return this.SendCtrlCmd('BACK')

    @abstractmethod            
    def ReadReg(this, regAddr, byteOffset, readCount, regValueList, regValueListStartIdx=0) : pass

    @abstractmethod            
    def ReadReg1Byte(this, regAddr, byteOffset) : pass

    @abstractmethod
    def WriteReg(this, regAddr, byteOffset, writeCount, regValueList, writeDataStartIdx=0) : pass

    @abstractmethod
    def WriteReg1Byte(this, regAddr, byteOffset, regValue) : pass
        
    @abstractmethod
    def WriteCtrlReg(this, regAddr) : pass

    #COMM_API Bool TedDD_DSIM_MipiReadReg(int dutIdx, int addr, int byteOffset, int readCount, unsigned char* buf, int bufMaxByteSize);
    @abstractmethod
    def DD_DSIM_MipiReadReg(this, dutIdx, regAddr, byteOffset, readCount) : pass

    #COMM_API unsigned char TedDD_DSIM_MipiReadReg1Byte(int dutIdx, int addr, int byteOffset);
    @abstractmethod
    def DD_DSIM_MipiReadReg1Byte(this, dutIdx, regAddr, byteOffset) : pass

    #COMM_API Bool TedDD_DSIM_MipiReadReg(int dutIdx, int addr, int byteOffset, int readCount, unsigned char* buf, int bufMaxByteSize);
    @abstractmethod
    def DD_DSIM_MipiWriteReg(this, dutIdx, regAddr, byteOffset, regValueList) : pass

    #COMM_API Bool TedDD_DSIM_MipiWriteReg1Byte(int dutIdx, int addr, int byteOffset, unsigned char data);
    @abstractmethod
    def DD_DSIM_MipiWriteReg1Byte(this, dutIdx, regAddr, byteOffset, regValue) : pass

    #WREG0=0x39, [Addr], [regVal0], [regVal1].....
    #COMM_API Bool TedDD_DSIM_MipiWriteReg39(int dutIdx, int addr, int writeCount, unsigned char* buf);
    @abstractmethod
    def DD_DSIM_MipiWriteReg39(this, dutIdx, regAddr, regValueList) : pass

    #WREG0=0x15, [Addr], [regVal]
    #COMM_API Bool TedDD_DSIM_MipiWriteReg15(int dutIdx, int addr, unsigned char value);
    @abstractmethod
    def DD_DSIM_MipiWriteReg15(this, dutIdx, regAddr, regValue) : pass

    #WREG0=0x05, [Addr]
    #COMM_API Bool TedDD_DSIM_MipiWriteReg05(int dutIdx, int addr);
    @abstractmethod
    def DD_DSIM_MipiWriteReg05(this, dutIdx, regAddr) : pass

    #WREG0=0x07, [value]   :   Compressd Mode Command
    #COMM_API Bool TedDD_DSIM_MipiWriteReg07(int dutIdx, int addr);
    @abstractmethod
    def DD_DSIM_MipiWriteReg07(this, dutIdx, value) : pass
        
    @abstractmethod
    def CatchPower(this, powerInfo) : pass

    @abstractmethod
    def PatternConnect(this) : pass

    @abstractmethod
    def PatternDisconnect(this) : pass

    @abstractmethod
    def PatternIsConnect(this) : pass

    @abstractmethod
    def PatternSetCommand(this, ptrnCmd) : pass

    @abstractmethod
    def PatternPaint(this, r, g, b, a) : pass

    @abstractmethod
    def PatternUpdateScreen(this) : pass

    @abstractmethod
    def PatternDrawImage(this, imgFileName) : pass

    @abstractmethod
    def PatternScreenVerify(this, r, g, b) : pass

    @abstractmethod
    def ANA670X_GetChipIDCount(this) : pass

    @abstractmethod
    def ANA670X_GetChipID(this, dutIdx) : pass

    @abstractmethod
    def ANA670X_GetProductRevisionBytesCount(this) : pass

    @abstractmethod
    def ANA670X_GetProductRevisionBytes(this, dutIdx) : pass

    @abstractmethod
    def ANA670X_SetFrameRate(this, dutIdx, fr) : pass

    @abstractmethod
    def ANA670X_GetFrameRate(this, dutIdx) : pass

    @abstractmethod
    def SysDelay(this, delay) : pass
        
    @abstractmethod
    def DebugMessage(this, msg) : pass
        
    @abstractmethod
    def DebugFuncEnter(this, funcName) : pass
    
    @abstractmethod
    def DebugFuncLeave(this, funcName) : pass
    
    @abstractmethod
    def DD_FB_blank(this, dutIdx, value) : pass
    @abstractmethod
    def DD_DSIM_manual_ctrl(this, dutIdx, value) : pass
    @abstractmethod
    def DD_DSIM_power_ctrl(this, dutIdx, value) : pass
    @abstractmethod
    def DD_DSIM_source_cal(this, dutIdx, value) : pass
    @abstractmethod
    def DD_DSIM_sleepin(this, dutIdx, value) : pass
    @abstractmethod
    def DD_DSIM_sleepout(this, dutIdx, value) : pass
    @abstractmethod
    def DD_DSIM_deep_standby(this, dutIdx, value) : pass
    @abstractmethod
    def DD_DSIM_displayon(this, dutIdx, value) : pass
    @abstractmethod
    def DD_DSIM_reset_ctrl(this, dutIdx, value) : pass

    #//VLIN1_ADC
    #TDEVICE_API TED_BOOL TDeviceVlin1AdcSetSamples(TDEVICE_HDL hdl, int value);  
    @abstractmethod
    def Vlin1AdcSetSamples(this, dutIdx, value) : pass

    #DEVICE_API TED_BOOL TDeviceVlin1AdcSetInterval(TDEVICE_HDL hdl, int value); 
    @abstractmethod
    def Vlin1AdcSetInterval(this, dutIdx, value) : pass

    #TDEVICE_API TED_BOOL TDeviceVlin1AdcSetChannelOn(TDEVICE_HDL hdl, int chIdx);
    @abstractmethod
    def Vlin1AdcSetChannelOn(this, dutIdx, value) : pass

    #TDEVICE_API TED_BOOL TDeviceVlin1AdcSetChannelOff(TDEVICE_HDL hdl, int chIdx); 
    @abstractmethod
    def Vlin1AdcSetChannelOff(this, dutIdx, value) : pass

    #TDEVICE_API TED_BOOL TDeviceVlin1AdcSetMode(TDEVICE_HDL hdl, int value);  
    @abstractmethod
    def Vlin1AdcSetMode(this, dutIdx, value) : pass

    #TDEVICE_API TED_BOOL TDeviceVlin1AdcGetVoltage(TDEVICE_HDL hdl, int chIdx);  
    @abstractmethod
    def Vlin1AdcGetVoltage(this, dutIdx, chIdx) : pass

    #TDEVICE_API TED_BOOL TDeviceVlin1AdcGetVoltageVLIN1(TDEVICE_HDL hdl);  
    @abstractmethod
    def Vlin1AdcGetVoltageVLIN1(this, dutIdx) : pass

    #TDEVICE_API TED_BOOL TDeviceVlin1AdcGetVoltageVBAT(TDEVICE_HDL hdl);  
    @abstractmethod
    def Vlin1AdcGetVoltageVBAT(this, dutIdx) : pass

    #TDEVICE_API TED_BOOL TDeviceVlin1AdcGetVoltageELVDD(TDEVICE_HDL hdl);  
    @abstractmethod
    def Vlin1AdcGetVoltageELVDD(this, dutIdx) : pass

    #TDEVICE_API TED_BOOL TDeviceVlin1AdcGetCurrent(TDEVICE_HDL hdl, int chIdx);  
    @abstractmethod
    def Vlin1AdcGetCurrent(this, dutIdx, chIdx) : pass

    #TDEVICE_API TED_BOOL TDeviceVlin1AdcGetCurrentVLIN1(TDEVICE_HDL hdl); 
    @abstractmethod
    def Vlin1AdcGetCurrentVLIN1(this, dutIdx) : pass

    #TDEVICE_API TED_BOOL TDeviceVlin1AdcGetCurrentVBAT(TDEVICE_HDL hdl); 
    @abstractmethod
    def Vlin1AdcGetCurrentVBAT(this, dutIdx) : pass

    #TDEVICE_API TED_BOOL TDeviceVlin1AdcGetCurrentELVDD(TDEVICE_HDL hdl); 
    @abstractmethod
    def Vlin1AdcGetCurrentELVDD(this, dutIdx) : pass

    #//VCI_ADC
    #TDEVICE_API TED_BOOL TDeviceVciAdcSetSamples(TDEVICE_HDL hdl, int value); 
    @abstractmethod
    def VciAdcSetSamples(this, dutIdx, value) : pass

    #TDEVICE_API TED_BOOL TDeviceVciAdcSetInterval(TDEVICE_HDL hdl, int value);
    @abstractmethod
    def VciAdcSetInterval(this, dutIdx, value) : pass

    #TDEVICE_API TED_BOOL TDeviceVciAdcSetChannelOn(TDEVICE_HDL hdl, int chIdx);
    @abstractmethod
    def VciAdcSetChannelOn(this, dutIdx, value) : pass

    #TDEVICE_API TED_BOOL TDeviceVciAdcSetChannelOff(TDEVICE_HDL hdl, int chIdx);
    @abstractmethod
    def VciAdcSetChannelOff(this, dutIdx, value) : pass

    #TDEVICE_API TED_BOOL TDeviceVciAdcSetMode(TDEVICE_HDL hdl, int value);
    @abstractmethod
    def VciAdcSetMode(this, dutIdx, value) : pass
        
    #TDEVICE_API int TDeviceVciAdcGetVoltage(TDEVICE_HDL hdl, int chIdx);
    @abstractmethod
    def VciAdcGetVoltage(this, dutIdx, chIdx) : pass

    #TDEVICE_API int TDeviceVciAdcGetVoltageVCI(TDEVICE_HDL hdl);
    @abstractmethod
    def VciAdcGetVoltageVCI(this, dutIdx) : pass

    #TDEVICE_API int TDeviceVciAdcGetVoltageVDDR(TDEVICE_HDL hdl);
    @abstractmethod
    def VciAdcGetVoltageVDDR(this, dutIdx) : pass

    #TDEVICE_API int TDeviceVciAdcGetVoltageVDDI(TDEVICE_HDL hdl);
    @abstractmethod
    def VciAdcGetVoltageVDDI(this, dutIdx) : pass

    #TDEVICE_API int TDeviceVciAdcGetCurrent(TDEVICE_HDL hdl, int chIdx);
    @abstractmethod
    def VciAdcGetCurrent(this, dutIdx, chIdx) : pass

    #DEVICE_API int TDeviceVciAdcGetCurrentVCI(TDEVICE_HDL hdl);
    @abstractmethod
    def VciAdcGetCurrentVCI(this, dutIdx) : pass

    #TDEVICE_API int TDeviceVciAdcGetCurrentVDDR(TDEVICE_HDL hdl);
    @abstractmethod
    def VciAdcGetCurrentVDDR(this, dutIdx) : pass

    #TDEVICE_API int TDeviceVciAdcGetCurrentVDDI(TDEVICE_HDL hdl);
    @abstractmethod
    def VciAdcGetCurrentVDDI(this, dutIdx) : pass

    #//SDOUT ADC
    #define TED_SDOUTADC_MAX_CH_COUNT 16
    #TDEVICE_API int TDeviceSoutAdcSetDevConfig(TDEVICE_HDL hdl, int value);
    @abstractmethod
    def SoutAdcSetDevConfig(this, dutIdx, value) : pass

    #TDEVICE_API int TDeviceSoutAdcSetInConfig(TDEVICE_HDL hdl, int chIdx,  int value); 
    @abstractmethod
    def SoutAdcSetInConfig(this, dutIdx, chIdx, value) : pass

    #COMM_API Bool TedAdcSoutSetRBSel(int dutIdx, int value)
    @abstractmethod
    def AdcSoutSetRBSel(this, dutIdx, value) : pass

    #COMM_API int TedAdcSoutGetRBSel(int dutIdx)
    @abstractmethod
    def AdcSoutGetRBSel(this, dutIdx) : pass

    #TDEVICE_API int TDeviceSoutAdcGetChannelCount(TDEVICE_HDL hdl);
    @abstractmethod
    def SoutAdcGetChannelCount(this, dutIdx) : pass

    #TDEVICE_API int TDeviceSoutAdcGetVoltage(TDEVICE_HDL hdl, int chIdx);  
    @abstractmethod
    def SoutAdcGetVoltage(this, dutIdx, chIdx) : pass

    #TDEVICE_API TED_BOOL TDeviceSoutAdcGetAllVoltage(TDEVICE_HDL hdl, int* voltageArray);
    @abstractmethod
    def SoutAdcGetAllVoltage(this, dutIdx) : pass

    #TDEVICE_API TED_BOOL TDeviceLdoAdcSetInConfig(TDEVICE_HDL hdl, int chIdx, int value);
    @abstractmethod
    def LdoAdcSetInConfig(this, dutIdx, chIdx, value) : pass

    #TDEVICE_API int TDeviceLdoAdcGetChannelCount(TDEVICE_HDL hdl);
    @abstractmethod
    def LdoAdcGetChannelCount(this, dutIdx) : pass

    #TDEVICE_API int TDeviceLdoAdcGetVoltage(TDEVICE_HDL hdl, int chIdx);
    @abstractmethod
    def LdoAdcGetVoltage(this, dutIdx, chIdx) : pass

    #TDEVICE_API TED_BOOL TDeviceLdoAdcGetAllVoltage(TDEVICE_HDL hdl, int* voltageArray);
    @abstractmethod
    def LdoAdcGetAllVoltage(this, dutIdx) : pass

    #TDEVICE_API TED_BOOL TDeviceRegAdcSetInConfig(TDEVICE_HDL hdl, int chIdx, int value);
    @abstractmethod
    def RegAdcSetInConfig(this, dutIdx, chIdx, value) : pass

    #TDEVICE_API int TDeviceRegAdcGetChannelCount(TDEVICE_HDL hdl);
    @abstractmethod
    def RegAdcGetChannelCount(this, dutIdx) : pass

    #TDEVICE_API int TDeviceRegAdcGetVoltage(TDEVICE_HDL hdl, int chIdx);
    @abstractmethod
    def RegAdcGetVoltage(this, dutIdx, chIdx) : pass

    #TDEVICE_API TED_BOOL TDeviceLdoAdcGetAllVoltage(TDEVICE_HDL hdl, int* voltageArray);
    @abstractmethod
    def RegAdcGetAllVoltage(this, dutIdx) : pass

    @abstractmethod            
    def AgingNotifyPyStart(this, pyFileName) : pass

    @abstractmethod            
    def AgingNotifyPyStop(this, pyFileName) : pass

    #COMM_API Bool TedAgingSetCurJobInfo(int dutIdx, int jobID, int status, int scIdx, int scCount, const char* desc);
    @abstractmethod
    def AgingSetCurJobInfo(this, dutIdx, jobID, status, scIdx, scCount, desc) : pass

    #COMM_API Bool TedAgingSetCurScInfo(int dutIdx, int scID, int status, int tcIdx, int tcCount, const char* desc);
    @abstractmethod
    def AgingSetCurScInfo(this, dutIdx, scID, status,tcIdx, tcCount, desc) : pass

    #COMM_API Bool TedAgingSetCurTcInfo(int dutIdx, int tcID, int status, int tcStepIdx, int tcStepCount, const char* desc);
    @abstractmethod
    def AgingSetCurTcInfo(this, dutIdx, tcID, status,tcStepIdx, tcStepCount, desc) : pass

    #COMM_API Bool TedAgingSetCurTcStepInfo(int dutIdx, int tcStepID, int status, const char* desc);
    @abstractmethod
    def AgingSetCurTcStepInfo(this, dutIdx, tcStepID, status, desc) : pass

    #TCOMM_API Bool TedAgingMeasureADC(int dutIdx, /*OUT*/void* res);
    @abstractmethod
    def AgingMeasureADC(this, dutIdx) : pass

    #COMM_API int TedAgingMeasureADCResultStructureByteSize();
    @abstractmethod
    def AgingMeasureADCResultStructureByteSize(this) : pass

#
# class TFileTransfer
#
class TFileTransfer :

    class Type(enum.IntEnum) : 
        T5 = 0
        
    class ErrorType(enum.IntEnum) : 
        Success = 0,
        SendPacket=1,
        NoResp=2,
        FileOpen=3,
        StorageSize=4,
        CRC=5
    
    #TDEVICE_API TFILETRANSFER_HDL TFileTransferCreate(enum TFileTransferType type, TDEVICE_HDL deviceHandle);
    def __init__(this, type, device) :
        this.__TFileTransferHandle = TFileTransferCreate(type, device.Handle)
        this.__FileName = ""

    def __getattr__(this, attrName) :
        if attrName == 'LastErrorString' : 
            return this.GetLastErrorString()
        if attrName == 'FileName' : 
            return this.__FileName
        else :
            raise AttributeError(attrName)


    #TDEVICE_API TED_BOOL TFileTransferDestroy(TFILETRANSFER_HDL fileTransferHandle);

    #TDEVICE_API TED_BOOL TFileTransferStart(TFILETRANSFER_HDL fileTransferHandle, const char* fileName);
    def Start(this, fileName) : 
        this.__FileName = fileName
        bytesString = fileName.encode('euc-kr')
        #bytesString = fileName.encode('ascii')
        #bytesString = fileName.encode('utf-8')
        ret = TFileTransferStart(this.__TFileTransferHandle, bytesString)
        return ret

    #TDEVICE_API TED_BOOL TFileTransferStop(TFILETRANSFER_HDL fileTransferHandle);
    def Stop(this) : 
        ret = TFileTransferStop(this.__TFileTransferHandle)
        return ret

    #TDEVICE_API int TFileTransferGetFileByteSize(TFILETRANSFER_HDL fileTransferHandle);
    def GetFileByteSize(this) : 
        ret = TFileTransferGetFileByteSize(this.__TFileTransferHandle)
        return ret

    #TDEVICE_API int TFileTransferGetTransferByteSize(TFILETRANSFER_HDL fileTransferHandle);
    def GetTransferByteSize(this) : 
        ret = TFileTransferGetTransferByteSize(this.__TFileTransferHandle)
        return ret

    #TDEVICE_API TED_BOOL TFileTransferIsStart(TFILETRANSFER_HDL fileTransferHandle);
    def IsStart(this) : 
        ret = TFileTransferIsStart(this.__TFileTransferHandle)
        return ret

    #TDEVICE_API TED_BOOL TFileTransferIsDone(TFILETRANSFER_HDL fileTransferHandle);
    def IsDone(this) : 
        ret = TFileTransferIsDone(this.__TFileTransferHandle)
        return ret

    #TDEVICE_API TED_BOOL TFileTransferIsError(TFILETRANSFER_HDL fileTransferHandle);
    def IsError(this) : 
        ret = TFileTransferIsError(this.__TFileTransferHandle)
        return ret

    #TDEVICE_API enum TFileTransferError TFileTransferGetLastError(TFILETRANSFER_HDL fileTransferHandle);
    def GetLastError(this) : 
        ret = TFileTransferGetLastError(this.__TFileTransferHandle)
        return ret

    def GetLastErrorString(this) :
        err = this.GetLastError()
        if err == TFileTransfer.ErrorType.Success :
            return "Success"
        elif err == TFileTransfer.ErrorType.SendPacket :
            return "SendPacket Error"
        elif err == TFileTransfer.ErrorType.NoResp :
            return "NoResp Error"
        elif err == TFileTransfer.ErrorType.FileOpen :
            return "FileOpen Error"
        elif err == TFileTransfer.ErrorType.StorageSize :
            return "StorageSize Error"
        elif err == TFileTransfer.ErrorType.CRC :
            return "CRC Error"
        else :
            return "Unknown Error"
    




from Anapass.TModule import *
import time

#
#  T5에 연결후,  Reset, Next수행후 ChipID를 읽는 동작을 3번 반복 
#
print("----------------------------------------------")
print("[anapass-python::Example] ReadChipID")
print("----------------------------------------------")

#TDevice 객체 생성, T5연결하는 Device객체이다.
device = TDevice(TDevice.Type.T5)

#
# Python프로그램을 Device를  연결하는 코드이다.
# T4/T5 에 연결해서 사용하기 위해서는, 먼저  TEDTools의 TMonitor가 활성화되어야 한다. 
# http://aits.anapass.com:8090/display/SI/TED+Tools  에서 UserManual Chap2. 'T4/T5구동기 연결하기' 참고 
#
print("Connect To " + device.GetName() )
isConnect = device.Connect()
if isConnect != True :
    print("Connect Fail")
    quit()

for loop in range(3) :

    print("----------------------------------------------")
    print("Loop :  ", loop )
    print("----------------------------------------------")

    #Reset
    print("Reset " + device.GetName())
    device.Reset()

    #1초 대기 
    print("sleep 1 second")
    time.sleep(1)

    #Next
    print("1st Next " + device.GetName())
    device.Next()

    #1초 대기 
    print("sleep 1 second")
    time.sleep(1)

    #Next
    print("2nd Next " + device.GetName())
    device.Next()

    #3초 대기 
    print("sleep 3 second")
    time.sleep(3)

    # ChipID를 읽는다.
    print("Read ChipID...")
    regAddr=0xD6 # ChipID
    byteOffset=0 # Level2Offset
    readCount=5  # 5바이트 
    regValueList=[0 for _ in range(readCount)]  #읽을 Register 값 개수만큼 리스트 할당 
    isOK = device.ReadReg(regAddr, byteOffset, readCount, regValueList)  #레지스터 Read함수 
    if isOK != True :
        print("FAIL: ReadReg,  Check the connection between TMonitor and T4/T5 Device")
        quit()
    # ChipID 읽은 결과 출력
    for regValue in regValueList :
        print(hex(regValue))
    print()


#연결을 끊는다.     
print("Disconnect from " + device.GetName())
device.Disonnect()

print("End of Exam. Bye!!")
print()








from Anapass.TModule import *

#
#  T5에 연결후,  ChipID를 읽고, 화면에 출력하는 예제 
#
print("----------------------------------------------")
print("[anapass-python::Example] File Transfer")
print("----------------------------------------------")

#TDevice 객체 생성, T5연결하는 Device객체이다.
device = TDevice(TDevice.Type.T5)

#
# Python프로그램을 Device를  연결하는 코드이다.
# T4/T5 에 연결해서 사용하기 위해서는, 먼저  TEDTools의 TMonitor가 활성화되어야 한다. 
# http://aits.anapass.com:8090/display/SI/TED+Tools  에서 UserManual Chap2. 'T4/T5구동기 연결하기' 참고 
#
print("Connect To " + device.GetName() )
isConnect = device.Connect()
assert isConnect 

# FileTransfer 객체 생성
fileTransfer = TFileTransfer(TFileTransfer.Type.T5, device)
assert fileTransfer

ret = fileTransfer.Start("d:\\work\\A2-655UV01-OTP-DEV190904-ANA6705_hthwang.TMD")
#ret = fileTransfer.Start("D:\Work\업무\AAA-업무 완성\2. Product\A2-655UV01-OTP-DEV190904-ANA6705_hthwang.TMD")
#ret = fileTransfer.Start("D:\Work\업무\AAA-업무 완성\2.Product\A2-655UV01-OTP-DEV190904-ANA6705_hthwang.TMD")
#ret = fileTransfer.Start("D:\\Work\\업무\\AAA-업무 완성\\2Product\\A2-655UV01-OTP-DEV190904-ANA6705_hthwang.TMD")
#ret = fileTransfer.Start("D:\\Work\업무\AAA-업무 완성\Product\A2-655UV01-OTP-DEV190904-ANA6705_hthwang.TMD")
#ret = fileTransfer.Start("D:\Work\업무\AAA-업무 완성\A2-655UV01-OTP-DEV190904-ANA6705_hthwang.TMD")
#ret = fileTransfer.Start("\\\\nas\\13_Data\\SoC\\SoC_SI\\업무공유\\2. Product\\ANA6705(FHD+)\\3. EEPROM_TMD_VEC\\TMD\\HHT\\TMD-전송-Test\\A2-655UV01-OTP-DEV190904-ANA6705_hht13.TMD")
#ret = fileTransfer.Start("\\\\nas\\13_Data\\SoC\\SoC_SI\\업무공유\\2. Product\\ANA6705(FHD+)\\3. EEPROM_TMD_VEC\\TMD\\HHT\\MAIN_V5_190624.dll")


#ret = fileTransfer.Start("T:\SoC\SoC_SI\업무공유\2. Product\ANA6705(FHD+)\3. EEPROM_TMD_VEC\TMD\HHT\TMD-전송-Test\A2-655UV01-OTP-DEV190904-ANA6705_hht13.TMD")
if ret != True :
    raise AssertionError(fileTransfer.LastErrorString + "  FileName: " + fileTransfer.FileName)

fileByteSize = fileTransfer.GetFileByteSize()
while True :
    transferSize = fileTransfer.GetTransferByteSize()
    if fileTransfer.IsDone() :
        print("File Transfer Success " )
        break
    elif fileTransfer.IsError() :
        print("File Transfer Error "  + fileTransfer.LastErrorString )
        break
    else :
        print("%d / %d " % (transferSize, fileByteSize) , end='\r' )


#연결을 끊는다.     
print("Disconnect from " + device.GetName() )
device.Disonnect()

print("End of Exam. Bye!!")
print()






from Anapass.TModule import *
from abc import ABCMeta, abstractmethod
import os
import sys
import getopt

class Base(metaclass=ABCMeta) :

    class Status(enum.IntEnum) : 
        Begin = 1
        End = 2

    class Item :
        def __init__(this, obj, loopCnt) : 
            this.__Obj = obj
            this.__loopCnt = loopCnt

        def Run(this) :
            for i in range(this.__loopCnt) :
                this.__Obj.Run()

        def GetObj(this) :
            return this.__Obj

    def __init__(this, device, dutIdx, id, desc) : 
        this.device = device
        this.ID = id;
        this.desc = desc
        this.__ItemList=[]
        this.DutIdx = dutIdx
        this.IsRunTcStep = True

    def Add(this, obj, loopCnt=1) :
        item = Base.Item(obj, loopCnt)
        this.__ItemList.append(item)

    def Begin(this) : this.__Begin__()
    def __Begin__(this) : pass
    def End(this) : this.__End__()
    def __End__(this) : pass
    def Run(this) :
        this.Begin()
        for item in this.__ItemList :
            item.Run()
        this.__Run__()
        this.End()
    def __Run__(this) : pass

    def GetItemList(this) :
        return this.__ItemList

    def GetItemCount(this) :
        return len(this.__ItemList)

    def GetID(this):
        return this.ID

    def SetRunTcStep(this, flag) :
        this.IsRunTcStep = flag
        for item in this.__ItemList :
            item.GetObj().SetRunTcStep(flag)

class TCStepBase(Base) : 

    def ParsingID(className) :
        assert className[0:7]=='TCStep_'
        szID = className[7:]
        ID=int(szID)
        return ID

    def __init__(this, classID, device, dutIdx, desc) : 
        
        if type(classID) == type('') :
            #print("type is String")
            Base.__init__(this, device, dutIdx, TCStepBase.ParsingID(classID), desc)   #className must be  format  'TCStep_XX' 
        elif type(classID) == type(1) : 
            #print("type is INT")
            Base.__init__(this, device, dutIdx, classID, desc)
        else :
            raise AssertionError()

        this._JobID = -1
        this._ScID = -1
        this._TcID = -1
        this._TcIndex = -1
        
        this._ScCount = 0
        this._TcCount = 0
        this._TcStepCount = 0


    def Begin(this) : 

        this.device.AgingSetCurTcStepInfo(this.DutIdx, this.ID, Base.Status.Begin.value, this.desc)
        sysErrFlag = this.device.SysGetErrFlag()

        print("-------------------------------------------------------------------------------------------------------------------------")
        print("TCStep[%d-(%d/%d)] : %s  JobID=%d ScID=%d/%d TcIdx=%d/%d  SysErrFlag=0x%016X" %(this._TcID, this.ID, this._TcStepCount, this.desc, this._JobID, this._ScID, this._ScCount, this._TcIndex, this._TcCount, sysErrFlag) )
        print("--------------------------------------------------------------------------------------------------------------------------")

        if this.IsRunTcStep : 
            this.__Begin__()
        
    
    def End(this) : 

        this.device.AgingMeasureADC(this.DutIdx)
        sysErrFlag = this.device.SysGetErrFlag()

        if this.IsRunTcStep : 
            this.__End__()

        print("");
        print("");
        this.device.AgingSetCurTcStepInfo(this.DutIdx, this.ID, Base.Status.End.value, this.desc)

    def Run(this) :
        this.Begin()

        sysErrFlag = this.device.SysGetErrFlag()
        if this.IsRunTcStep  : 
            this.__Run__()
        else :
            print("TCStep[%d-(%d/%d)] : SKIP  SysErrFlag=0x%016X" %(this._TcID, this.ID, this._TcStepCount, sysErrFlag) )

        this.End()

    def SetJobID(this, jobID) :
        this._JobID = jobID
    
    def SetScID(this, scID) :
        this._ScID = scID
    
    def SetTcID(this, tcID) :
        this._TcID = tcID

    def SetTcIndex(this, tcIdx) :
        this._TcIndex = tcIdx

    def SetScCount(this, scCnt) :
        this._ScCount = scCnt

    def SetTcCount(this, tcCnt) :
        this._TcCount = tcCnt

    def SetTcStepCount(this, tcStepCount) :
        this._TcStepCount = tcStepCount
    

class TCBase(Base) : 

    def ParsingID(pyFileName) :
        baseName = os.path.basename(pyFileName)
        assert baseName[0]=='T'
        assert baseName[1]=='C'
        assert baseName[2]=='_'
        szID = baseName[3:7]
        assert baseName[7]=='.'
        assert baseName[8]=='p'
        assert baseName[9]=='y'
        ID=int(szID)
        return ID

    def __init__(this, pyFileName, device, dutIdx, desc) : 
        Base.__init__(this, device, dutIdx, TCBase.ParsingID(pyFileName), desc)
        this.__List=[]

        this._JobID = -1
        this._ScID = -1
        this._ScCount = 0
        this._TcCount = 0
        this._TcIndex = 0

        this._IsRunAvailable = True
         
    def Begin(this) : 

        this.device.AgingSetCurTcInfo(this.DutIdx, this.ID, Base.Status.Begin.value, 0, this.GetItemCount(), this.desc)
        sysErrFlag = this.device.SysGetErrFlag()

        print("***********************************************************************************")
        print("TC_%04d Begin : TcStepCnt=%d  %s  Job=%d Sc=%d tcCnt=%d  SysErr=0x%016X" %(this.ID, this.GetItemCount(), this.desc, this._JobID, this._ScID, this._TcCount, sysErrFlag) )
        
        tcStepList = this.GetItemList()
        for tcStepItem in tcStepList :
            tcStepItem.GetObj().SetJobID(this._JobID)
            tcStepItem.GetObj().SetScID(this._ScID)
            tcStepItem.GetObj().SetScCount(this._ScCount)
            tcStepItem.GetObj().SetTcID(this.GetID())
            tcStepItem.GetObj().SetTcIndex(this._TcIndex)
            tcStepItem.GetObj().SetTcCount(this._TcCount)
            tcStepItem.GetObj().SetTcStepCount(this.GetItemCount())
        

        if this.IsRunTcStep : 
            this.__Begin__()

    
    def End(this) : 

        sysErrFlag = this.device.SysGetErrFlag()
        if this.IsRunTcStep : 
            this.__End__()
        
        print("TC_%04d End : TcStepCnt=%d  %s  sysErrFlag=0x%016X" %(this.ID, this.GetItemCount(), this.desc, sysErrFlag) )
        print("***********************************************************************************")
        print()

        this.device.AgingSetCurTcInfo(this.DutIdx, this.ID, Base.Status.End.value, this.GetItemCount()-1, this.GetItemCount(), this.desc)
    
    def SetJobID(this, jobID) :
        this._JobID = jobID

    def SetScID(this, scID) :
        this._ScID = scID

    def SetTcIndex(this, tcIdx) :
        this._TcIndex = tcIdx
    
    def SetTcCount(this, tcCount) :
        this._TcCount = tcCount
    
    def SetScCount(this, scCount) :
        this._ScCount = scCount
    
        
class SCBase(Base) : 

    def ParsingID(pyFileName) :
        baseName = os.path.basename(pyFileName)
        assert baseName[0]=='S'
        assert baseName[1]=='C'
        assert baseName[2]=='_'
        szID = baseName[3:7]
        assert baseName[7]=='.'
        assert baseName[8]=='p'
        assert baseName[9]=='y'
        ID=int(szID)
        return ID


    def __init__(this, classID, device, dutIdx, desc) : 
        
        if type(classID) == type('') :
            #print("type is String")
            Base.__init__(this, device, dutIdx, SCBase.ParsingID(classID), desc)   #className must be  format  'TCStep_XX' 
        elif type(classID) == type(1) : 
            #print("type is INT")
            Base.__init__(this, device, dutIdx, classID, desc)
        else :
            raise AssertionError()

        this.__List=[]
        this._JobID = -1
        this._ScCount = 0
    
    def Begin(this) : 
        print("###############################################################################################################")
        print("SC_%04d Begin: TcCnt=%d %s  JobID=%d ScCnt=%d" %(this.ID, this.GetItemCount(), this.desc, this._JobID, this._ScCount) )
        
        tcIdx = 0
        tcStepList = this.GetItemList()
        for tcStepItem in tcStepList :
            tcStepItem.GetObj().SetJobID(this._JobID)
            tcStepItem.GetObj().SetScID(this.GetID())
            tcStepItem.GetObj().SetScCount(this._ScCount)
            tcStepItem.GetObj().SetTcIndex(tcIdx)
            tcStepItem.GetObj().SetTcCount(this.GetItemCount())

            tcIdx = tcIdx +1 


        this.__Begin__()
        this.device.AgingSetCurScInfo(this.DutIdx, this.ID, Base.Status.Begin.value, 0, this.GetItemCount(), this.desc)


    def End(this) : 
        this.device.AgingSetCurScInfo(this.DutIdx, this.ID, Base.Status.End.value, this.GetItemCount()-1, this.GetItemCount(), this.desc)
        this.__End__()
                
        print("SC_%04d End: TcCnt=%d %s  JobID=%d ScCnt=%d" %(this.ID, this.GetItemCount(), this.desc, this._JobID, this._ScCount) )
        print("###############################################################################################################")
        print()
        print()
        
    def SetJobID(this, jobID) :
        this._JobID = jobID

    def SetScCount(this, scCount) :
        this._ScCount = scCount
    

class JobBase(Base) : 
    
    def ParsingID(pyFileName) :
        baseName = os.path.basename(pyFileName)
        assert baseName[0]=='J'
        assert baseName[1]=='O'
        assert baseName[2]=='B'
        assert baseName[3]=='_'
        szID = baseName[4:8]
        ID=int(szID)
        assert baseName[8]=='.'
        assert baseName[9]=='p'
        assert baseName[10]=='y'
        return ID

    def __init__(this, pyFileName, device, dutIdx, desc) : 
        Base.__init__(this, device, dutIdx, JobBase.ParsingID(pyFileName), desc)
        this.__List=[]

    def Begin(this) : 
        print()
        print()
        print("//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////")
        print("JOB_%04d Begin: ScCnt=%d  %s" %(this.ID, this.GetItemCount(), this.desc) )
        
        tcStepList = this.GetItemList()
        for tcStepItem in tcStepList :
            tcStepItem.GetObj().SetJobID(this.GetID())
            tcStepItem.GetObj().SetScCount(this.GetItemCount())

        this.__Begin__()
        this.device.AgingSetCurJobInfo(this.DutIdx, this.ID, Base.Status.Begin.value, 0, this.GetItemCount(), this.desc)

    def End(this) : 
        this.__End__()
        this.device.AgingSetCurJobInfo(this.DutIdx, this.ID, Base.Status.End.value, this.GetItemCount()-1, this.GetItemCount(), this.desc)

        print("-------------------------------------------------")
        print("JOB_%04d End: ScCnt=%d  %s" %(this.ID, this.GetItemCount(), this.desc) )
        print("//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////")
        print()
        print()
        print()
        

class App :

    def print_help() :
        print(" ")
        print("py TC_xxxx.py --boardid=<boardid> ")
        print(" ")
        print("   <options-name>")
        print("       --boardid:  execute board id")
        print(" ")

    def __init__(this, pyFileName, argv) : 

        try :
            # opts : getopt 옵션에 따라 파싱 ex) [('-i', 'myin')]
            # etc_args: getopt 옵션 이외에 입력된 일반 Argument
            # argv 첫번째 (index:0) 는 Python파일명, 두번째 (index:1) 부터  Arguments
            opts, etc_args = getopt.getopt(sys.argv[1:], "hb:", ["help", "boardid=" ])

        except getopt.GetoptError:  #옵션지정이 올바르지 않은 경우
            print("ERROR :  the name of option is invalid")
            sys.exit()

        this._PyFileName = os.path.basename(pyFileName)
        this.__BoardID = -1
        for opt, arg in opts :
            if opt in ("-h", "--help") :
                App.print_help()
                sys.exit();

            elif opt in ("-b", "--boardid") :
                this.__BoardID = int(arg)

    def GetBoardID(this) :
        return this.__BoardID;

    def GetDevice(this) :
        return this.__TDevice

    def Begin(this) : 
        #TDevice Create
        this.__TDevice = TDevice(TDevice.Type.TESys)
        this.__TDevice.SysSetServerIPAddr("127.0.0.1")  
        
        print("Connect To " + this.__TDevice.GetName() )
    
        #TDevice Connect
        isConnect = this.__TDevice.Connect()
        assert isConnect 

        #if user set BoardID, it sends BoardID to TESys-Python-Module 
        if(this.__BoardID > 0) :
            print("SetBoardID  %d" % this.__BoardID )
            this.__TDevice.SysSetBoardID(this.__BoardID)

        #Notify PyStart
        this.__TDevice.AgingNotifyPyStart(this._PyFileName)


    def End(this) :

        #Notify PyStop
        this.__TDevice.AgingNotifyPyStop(this._PyFileName)

        #Disconnect
        print("Disconnect from " + this.__TDevice.GetName() )
        this.__TDevice.Disonnect()
        
        
        print("----------------------------------------------------------------");
        print("MeasourceAdCCount = %d"%this.__TDevice.GetAgingMeasureADCCount())
        print("----------------------------------------------------------------");

        print("End of Exam. Bye!!")
        print()

        







from Anapass.TModule import *

###########################################################################################
#
#  T5에 연결후,  
#    Test1. ChipID(0xD6) 2nd Parm값을 읽고,  1 더한값을 Write/Verify 한다.
#    Test2. ChiID 3rd/4th/5th Parm값을 읽고,  1 더한값을 Write/Verify 한다. 
#
###########################################################################################
print("----------------------------------------------")
print("[anapass-python::Example] Register Read/Write/Verify")
print("----------------------------------------------")

###########################################################################################
#
#TDevice 객체 생성, T5연결하는 Device객체이다.
#
###########################################################################################
device = TDevice(TDevice.Type.T5)
#device = TDevice(TDevice.Type.TESys)

###########################################################################################
#
# Python프로그램을 Device를  연결하는 코드이다.
# T4/T5 에 연결해서 사용하기 위해서는, 먼저  TEDTools의 TMonitor가 활성화되어야 한다. 
# http://aits.anapass.com:8090/display/SI/TED+Tools  에서 UserManual Chap2. 'T4/T5구동기 연결하기' 참고 
#
###########################################################################################
print("Connect To " + device.GetName() )
isConnect = device.Connect()
if isConnect != True :
    print("Connect Fail")
    quit()

###########################################################################################
#
# Test1. ChipID(0xD6) 2nd Parm값을 읽고,  1 더한값을 Write/Verify 한다.
# 
###########################################################################################
regAddr=0xD6    # ChipID
byteOffset = 1  #2nd Parm

print("Test1. RegAddr=", hex(regAddr), " 2nd Parm R/W" )
regValue = device.ReadReg1Byte(regAddr, byteOffset)  
if regValue == -1 :
    print("FAIL: ReadReg1Byte,  Check the connection between TMonitor and T4/T5 Device")
    quit()

print("\tRead: ", hex(regValue))

regValue = (regValue+1)%0xFF
isOK=device.WriteReg1Byte(regAddr, byteOffset, regValue)
if isOK != True :
    print("FAIL: ReadReg,  Check the connection between TMonitor and T4/T5 Device")
    quit()
print("\tWrite: ", hex(regValue))

regValue = 0
regValue = device.ReadReg1Byte(regAddr, byteOffset)  
if regValue == -1 :
    print("FAIL: ReadReg1Byte,  Check the connection between TMonitor and T4/T5 Device")
    quit()
print("\tVerify: ", hex(regValue))
print()


###########################################################################################
#
# Test2. ChiID 3rd/4th/5th Parm값을 읽고,  1 더한값을 Write/Verify 한다. 
# 
###########################################################################################
print("Test1. RegAddr=", hex(regAddr), " 3rd/4th/5th Parm R/W" )
regAddr=0xD6 # ChipID
byteOffset=2 # byteOffset  3rd
readCount=3  # 읽을 개수 
regValueList=[0 for _ in range(readCount)]  #읽을 Register 값 개수만큼 리스트 할당 
isOK = device.ReadReg(regAddr, byteOffset, readCount, regValueList)  #레지스터 Read함수 
if isOK != True :
    print("FAIL: ReadReg,  Check the connection between TMonitor and T4/T5 Device")
    quit()
print("\tRead: ", str.join("", ("0x%02X " %i for i in regValueList)) )

#값을 1씩 더한다
for idx, regValue in enumerate(regValueList) :
     regValue += 1
     regValue %=256
     regValueList[idx] = regValue

isOK = device.WriteReg(regAddr, byteOffset, readCount, regValueList) 
if isOK != True :
    print("FAIL: WriteReg,  Check the connection between TMonitor and T4/T5 Device")
    quit()
print("\tWrite: ", str.join("", ("0x%02X " %i for i in regValueList)) )

regValueVerifyList=[0 for _ in range(readCount)]  #읽을 Register 값 개수만큼 리스트 할당 
isOK = device.ReadReg(regAddr, byteOffset, readCount, regValueVerifyList)  #레지스터 Read함수 
if isOK != True :
    print("FAIL: ReadReg,  Check the connection between TMonitor and T4/T5 Device")
    quit()
print("\tVerify: ", str.join("", ("0x%02X " %i for i in regValueVerifyList)) )
print()

###########################################################################################
#
#연결을 끊는다.     
#
###########################################################################################
print("Disconnect from " + device.GetName())
device.Disonnect()

###########################################################################################
#
# 프로그램종료
#
###########################################################################################
print("End of Exam. Bye!!")
print()





from Anapass.Example import ReadChipID
from Anapass.Example import ResetNext
from Anapass.Example import RegReadWriteVerify
from Anapass.Example import CatchPower
from Anapass.Example import SendTxtCmd






from ctypes import *
from ctypes.wintypes import *
from enum import Enum
import time
import platform
import sys
import os
import pkg_resources  # part of setuptools
import enum
import struct
from abc import ABCMeta, abstractmethod
from Anapass  import TDeviceWinDLL
from Anapass.TDeviceBase  import *

#
# class TDevice
#
class TDeviceWin(TDeviceBase) :

    class ErrorString(enum.Enum) :
        GetResp="ErrorGetResp"
    
    def __init__(this, deviceTypeValue):

        TDeviceBase.__init__(this, deviceTypeValue)
        this.__Api = TDeviceWinDLL.Api()
        this.__Api.TSys_WinInit();

        #print(DisplayName +"TRY: create " + deviceType.name )
        this.__DeviceHandle = this.__Api.TDeviceCreate(deviceTypeValue)
        
        #struct TED_POWER_INFO {
        #    TED_S32 no;
        #    TED_S32 available[10];
        #    TED_S32 value1[10];
        #    TED_FLOAT fV[10];
        #    TED_FLOAT fA[10];
        #    TED_FLOAT fRange1[10];
        #    TED_FLOAT fRange2[10];dir

        #};

        this.__PowerStructFmt = 'i'    #    TED_S32 no;    
        this.__PowerStructFmt+='10i'   # TED_S32 available[10];
        this.__PowerStructFmt+='10i'   # TED_S32 value[10];
        this.__PowerStructFmt+='10f'   # TED_FLOAT fV[10];
        this.__PowerStructFmt+='10f'   # TED_FLOAT fV[10];
        this.__PowerStructFmt+='10f'   # TED_FLOAT fRange1[10];
        this.__PowerStructFmt+='10f'   # TED_FLOAT fRange2[10];
        
        # arg=list(range(61))

        this.__PowerStructData = struct.pack(this.__PowerStructFmt, 0,
                           0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                           0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                           0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 
                           0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 
                           0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 
                           0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 
                           )

    def __del__(this):
        #print("TDevice::~TDevice")
        this.__Api.TDeviceDestroy(this.__DeviceHandle)

            
    def SysSetServerIPAddr(this, serverIPAddr) :
        ret = this.__Api.TDeviceSysSetServerIPAddr(this.__DeviceHandle, TString.ConvertToCTypeStrng(serverIPAddr))
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    def SysSetBoardID(this, boardID) : 
        ret = this.__Api.TDeviceSysSetBoardID(this.__DeviceHandle, boardID)
        if ret==1 :
            bflag = True
        else :
            bflag = False
        return bflag

    def SetTcLocalSave(this, boardID, bFlag) : 
        return True #ted.SysSetTcLocalSave(boardID, bFlag)
        
    def Connect(this) :
        ret = this.__Api.TDeviceConnect(this.__DeviceHandle)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    def Disonnect(this) :
        ret = this.__Api.TDeviceDisconnect(this.__DeviceHandle)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag
    
    def SendTxtCmd(this, cmd) :
        #print("[this.__Api.TDevice.SendTxtCmd] " + cmd)
        ret = this.__Api.TDeviceSendTxtCmd(this.__DeviceHandle, TString.ConvertToCTypeStrng(cmd), c_char_p(0), 0, 10)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    def SendTxtCmdReadResp(this, cmd, maxRespByteSize) :
        respBytesArray=bytes(maxRespByteSize)
        ret = this.__Api.TDeviceSendTxtCmd(this.__DeviceHandle, TString.ConvertToCTypeStrng(cmd), respBytesArray, maxRespByteSize, 1000)
        if ret==1 :
            bflag = True
            resp = TString.ConvertCTypeStringToUnicode(respBytesArray)
        else :
            bflag = False;
            resp = this.__Api.TDevice.ErrorString.GetResp
        return resp

    #private methond
    def SendCtrlCmd(this, cmd) :  
        ret = this.__Api.TDeviceSendCtrlCmd(this.__DeviceHandle, TString.ConvertToCTypeStrng(cmd), c_char_p(0), 0, 0)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    def Reset(this) :
        return this.__SendCtrlCmd('RESET')

    def Next(this) :
        return this.__SendCtrlCmd('NEXT')

    def Back(this) :
        return this.__SendCtrlCmd('BACK')

    def ReadReg(this, regAddr, byteOffset, readCount, regValueList, regValueListStartIdx=0) :
        regValueInt = 0
        regValueByteArray=bytes(readCount)
        ret = this.__Api.TDeviceDD_DSIM_MipiReadReg(this.__DeviceHandle, regAddr, byteOffset, readCount, regValueByteArray)
        if ret==1 :
            for idx, regValueByte in enumerate(regValueByteArray) :
                regValueInt = regValueByte
                regValueInt &= 0xFF
                regValueList[idx + regValueListStartIdx] = regValueInt
            bflag = True
        else :
            bflag = False;
        return bflag

    def ReadReg1Byte(this, regAddr, byteOffset) :
        regValueArray=bytes(1)
        ret = this.__Api.TDeviceDD_DSIM_MipiReadReg1Byte(this.__DeviceHandle, regAddr, byteOffset, regValueArray)
        if ret==1 :
            regValue = regValueArray[0]
            regValue &= 0xFF
        else :
            regValue = -1
        return regValue

    def WriteReg(this, regAddr, byteOffset, writeCount, regValueList, writeDataStartIdx=0) :

        if writeDataStartIdx == 0 :
            regValueByteArray=bytes(regValueList)
        else :
            listTmp = regValueList[writeDataStartIdx:(writeDataStartIdx+writeCount)]
            regValueByteArray=bytes(listTmp)

        ret = this.__Api.TDeviceDD_DSIM_MipiWriteReg(this.__DeviceHandle, regAddr, byteOffset, writeCount, regValueByteArray)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    def WriteReg1Byte(this, regAddr, byteOffset, regValue) :
        ret = this.__Api.TDeviceDD_DSIM_MipiWriteReg1Byte(this.__DeviceHandle, regAddr, byteOffset, c_char(regValue))
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    def WriteCtrlReg(this, regAddr) :
        return this.WriteReg1Byte(regAddr, 0, 1)


    
    def CatchPower(this, powerInfo) :

        ret = this.__Api.TDeviceCatchPowerInfo(this.__DeviceHandle,  this.__PowerStructData, 1000)

        result= struct.unpack(this.__PowerStructFmt, this.__PowerStructData)

        resIdx=0
        
        powerInfo.No = result[resIdx] 
        resIdx += 1

        for i in range(10) :
            powerInfo.Avail[i] = result[i+resIdx]
        resIdx += 10

        for i in range(10) :
            powerInfo.Value1[i] = result[i+resIdx]
        resIdx += 10

        for i in range(10) :
            powerInfo.Voltage[i] = result[i+resIdx]
        resIdx += 10

        for i in range(10) :
            powerInfo.Current[i] = result[i+resIdx]
        resIdx += 10

        for i in range(10) :
            powerInfo.Range1[i] = result[i+resIdx]
        resIdx += 10

        for i in range(10) :
            powerInfo.Range2[i] = result[i+resIdx]
        resIdx += 10

        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    
    
    def DebugMessage(this, msg) :
        #print(msg)
        ret = this.__Api.TDeviceDebugMessage(this.__DeviceHandle, TString.ConvertToCTypeStrng(msg))
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    def DebugFuncEnter(this, funcName) :
        ret = this.__Api.TDeviceDebugFuncEnter(this.__DeviceHandle, TString.ConvertToCTypeStrng(funcName))
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    def DebugFuncLeave(this, funcName) :
        ret = this.__Api.TDeviceDebugFuncLeave(this.__DeviceHandle, TString.ConvertToCTypeStrng(funcName))
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag
            
    #//VLIN1_ADC
    #TDEVICE_API TED_BOOL this.__Api.TDeviceVlin1AdcSetSamples(TDEVICE_HDL hdl, int value);  
    def Vlin1AdcSetSamples(this, dutIdx, value) :
        ret = this.__Api.TDeviceVlin1AdcSetSamples(this.__DeviceHandle, dutIdx, value)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    #DEVICE_API TED_BOOL TDeviceVlin1AdcSetInterval(TDEVICE_HDL hdl, int value); 
    def Vlin1AdcSetInterval(this, dutIdx, value) :
        ret = this.__Api.TDeviceVlin1AdcSetInterval(this.__DeviceHandle, dutIdx, value)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    #TDEVICE_API TED_BOOL TDeviceVlin1AdcSetChannelOn(TDEVICE_HDL hdl, int chIdx);
    def Vlin1AdcSetChannelOn(this, dutIdx, value) :
        ret = this.__Api.TDeviceVlin1AdcSetChannelOn(this.__DeviceHandle, dutIdx, value)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    #TDEVICE_API TED_BOOL TDeviceVlin1AdcSetChannelOff(TDEVICE_HDL hdl, int chIdx); 
    def Vlin1AdcSetChannelOff(this, dutIdx, value) :
        ret = this.__Api.TDeviceVlin1AdcSetChannelOff(this.__DeviceHandle, dutIdx, value)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    #TDEVICE_API TED_BOOL TDeviceVlin1AdcSetMode(TDEVICE_HDL hdl, int value);  
    def Vlin1AdcSetMode(this, dutIdx, value) :
        ret = this.__Api.TDeviceVlin1AdcSetMode(this.__DeviceHandle, dutIdx, value)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    #TDEVICE_API TED_BOOL TDeviceVlin1AdcGetVoltage(TDEVICE_HDL hdl, int chIdx);  
    def Vlin1AdcGetVoltage(this, dutIdx, chIdx) :
        return this.__Api.TDeviceVlin1AdcGetVoltage(this.__DeviceHandle, dutIdx, chIdx)

    #TDEVICE_API TED_BOOL TDeviceVlin1AdcGetVoltageVLIN1(TDEVICE_HDL hdl);  
    def Vlin1AdcGetVoltageVLIN1(this, dutIdx) :
        return this.__Api.TDeviceVlin1AdcGetVoltageVLIN1(this.__DeviceHandle, dutIdx)

    #TDEVICE_API TED_BOOL TDeviceVlin1AdcGetVoltageVBAT(TDEVICE_HDL hdl);  
    def Vlin1AdcGetVoltageVBAT(this, dutIdx) :
        return this.__Api.TDeviceVlin1AdcGetVoltageVBAT(this.__DeviceHandle, dutIdx)

    #TDEVICE_API TED_BOOL TDeviceVlin1AdcGetVoltageELVDD(TDEVICE_HDL hdl);  
    def Vlin1AdcGetVoltageELVDD(this, dutIdx) :
        return this.__Api.TDeviceVlin1AdcGetVoltageELVDD(this.__DeviceHandle, dutIdx)

    #TDEVICE_API TED_BOOL TDeviceVlin1AdcGetCurrent(TDEVICE_HDL hdl, int chIdx);  
    def Vlin1AdcGetCurrent(this, dutIdx, chIdx) :
        return this.__Api.TDeviceVlin1AdcGetCurrent(this.__DeviceHandle, dutIdx, chIdx)

    #TDEVICE_API TED_BOOL TDeviceVlin1AdcGetCurrentVLIN1(TDEVICE_HDL hdl); 
    def Vlin1AdcGetCurrentVLIN1(this, dutIdx) :
        return this.__Api.TDeviceVlin1AdcGetCurrentVLIN1(this.__DeviceHandle, dutIdx)

    #TDEVICE_API TED_BOOL TDeviceVlin1AdcGetCurrentVBAT(TDEVICE_HDL hdl); 
    def Vlin1AdcGetCurrentVBAT(this, dutIdx) :
        return this.__Api.TDeviceVlin1AdcGetCurrentVBAT(this.__DeviceHandle, dutIdx)

    #TDEVICE_API TED_BOOL TDeviceVlin1AdcGetCurrentELVDD(TDEVICE_HDL hdl); 
    def Vlin1AdcGetCurrentELVDD(this, dutIdx) :
        return this.__Api.TDeviceVlin1AdcGetCurrentELVDD(this.__DeviceHandle, dutIdx)

    #//VCI_ADC
    #TDEVICE_API TED_BOOL TDeviceVciAdcSetSamples(TDEVICE_HDL hdl, int value); 
    def VciAdcSetSamples(this, dutIdx, value) :
        ret = this.__Api.TDeviceVciAdcSetSamples(this.__DeviceHandle, dutIdx, value)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    #TDEVICE_API TED_BOOL TDeviceVciAdcSetInterval(TDEVICE_HDL hdl, int value);
    def VciAdcSetInterval(this, dutIdx, value) :
        ret = this.__Api.TDeviceVciAdcSetInterval(this.__DeviceHandle, dutIdx, value)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    #TDEVICE_API TED_BOOL TDeviceVciAdcSetChannelOn(TDEVICE_HDL hdl, int chIdx);
    def VciAdcSetChannelOn(this, dutIdx, value) :
        ret = this.__Api.TDeviceVciAdcSetChannelOn(this.__DeviceHandle, dutIdx, value)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    #TDEVICE_API TED_BOOL TDeviceVciAdcSetChannelOff(TDEVICE_HDL hdl, int chIdx);
    def VciAdcSetChannelOff(this, dutIdx, value) :
        ret = this.__Api.TDeviceVciAdcSetChannelOff(this.__DeviceHandle, dutIdx, value)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    #TDEVICE_API TED_BOOL TDeviceVciAdcSetMode(TDEVICE_HDL hdl, int value);
    def VciAdcSetMode(this, dutIdx, value) :
        ret = this.__Api.TDeviceVciAdcSetMode(this.__DeviceHandle, dutIdx, value)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    #TDEVICE_API int TDeviceVciAdcGetVoltage(TDEVICE_HDL hdl, int chIdx);
    def VciAdcGetVoltage(this, dutIdx, chIdx) :
        return this.__Api.TDeviceVciAdcGetVoltage(this.__DeviceHandle, dutIdx, chIdx)

    #TDEVICE_API int TDeviceVciAdcGetVoltageVCI(TDEVICE_HDL hdl);
    def VciAdcGetVoltageVCI(this, dutIdx) :
        return this.__Api.TDeviceVciAdcGetVoltageVCI(this.__DeviceHandle, dutIdx)

    #TDEVICE_API int TDeviceVciAdcGetVoltageVDDR(TDEVICE_HDL hdl);
    def VciAdcGetVoltageVDDR(this, dutIdx) :
        return this.__Api.TDeviceVciAdcGetVoltageVDDR(this.__DeviceHandle, dutIdx)

    #TDEVICE_API int TDeviceVciAdcGetVoltageVDDI(TDEVICE_HDL hdl);
    def VciAdcGetVoltageVDDI(this, dutIdx) :
        return this.__Api.TDeviceVciAdcGetVoltageVDDI(this.__DeviceHandle, dutIdx)

    #TDEVICE_API int TDeviceVciAdcGetCurrent(TDEVICE_HDL hdl, int chIdx);
    def VciAdcGetCurrent(this, dutIdx, chIdx) :
        return this.__Api.TDeviceVciAdcGetCurrent(this.__DeviceHandle, dutIdx, chIdx)

    #DEVICE_API int TDeviceVciAdcGetCurrentVCI(TDEVICE_HDL hdl);
    def VciAdcGetCurrentVCI(this, dutIdx) :
        return this.__Api.TDeviceVciAdcGetCurrentVCI(this.__DeviceHandle, dutIdx)

    #TDEVICE_API int TDeviceVciAdcGetCurrentVDDR(TDEVICE_HDL hdl);
    def VciAdcGetCurrentVDDR(this, dutIdx) :
        return this.__Api.TDeviceVciAdcGetCurrentVDDR(this.__DeviceHandle, dutIdx)

    #TDEVICE_API int TDeviceVciAdcGetCurrentVDDI(TDEVICE_HDL hdl);
    def VciAdcGetCurrentVDDI(this, dutIdx) :
        return this.__Api.TDeviceVciAdcGetCurrentVDDI(this.__DeviceHandle, dutIdx)

    #//SDOUT ADC
    #define TED_SDOUTADC_MAX_CH_COUNT 16
    #TDEVICE_API int TDeviceSoutAdcSetDevConfig(TDEVICE_HDL hdl, int value);
    def SoutAdcSetDevConfig(this, dutIdx, value) :
        ret = this.__Api.TDeviceSoutAdcSetDevConfig(this.__DeviceHandle, dutIdx, value)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    #TDEVICE_API int TDeviceSoutAdcSetInConfig(TDEVICE_HDL hdl, int chIdx,  int value); 
    def SoutAdcSetInConfig(this, chIdx, dutIdx, value) :
        ret = this.__Api.TDeviceSoutAdcSetInConfig(this.__DeviceHandle, dutIdx, chIdx, value)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    #DEVICE_API int TDeviceSoutAdcSetRBSel(TDEVICE_HDL hdl, int value);    
    def SoutAdcSetRBSel(this, dutIdx, value) :
        ret = this.__Api.TDeviceSoutAdcSetRBSel(this.__DeviceHandle, dutIdx, value)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    #TDEVICE_API int TDeviceSoutAdcGetChannelCount(TDEVICE_HDL hdl);
    def SoutAdcGetChannelCount(this, dutIdx) :
        return this.__Api.TDeviceSoutAdcGetChannelCount(this.__DeviceHandle, dutIdx)

    #TDEVICE_API int TDeviceSoutAdcGetVoltage(TDEVICE_HDL hdl, int chIdx);  
    def SoutAdcGetVoltage(this, dutIdx, chIdx) :
        return this.__Api.TDeviceSoutAdcGetVoltage(this.__DeviceHandle, dutIdx, chIdx)

    #TDEVICE_API TED_BOOL TDeviceSoutAdcGetAllVoltage(TDEVICE_HDL hdl, int* voltageArray);
    def SoutAdcGetAllVoltage(this, dutIdx) :

        structFmt = 'i'    #    TED_S32 no;    
        for i in range(15) :
            structFmt += 'i'    #    TED_S32 no;    
        structData = struct.pack(structFmt, 
                           0, 0, 0, 0, 0, 0, 0, 0,
                           0, 0, 0, 0, 0, 0, 0, 0,
                           )
        voltageList=[-1 for _ in range(16)]
        ret = this.__Api.TDeviceSoutAdcGetAllVoltage(this.__DeviceHandle, dutIdx, structData)
        if ret==1 :
            result= struct.unpack(structFmt, structData)
            for i in range(16) :
                voltageList[i] = result[i]
        return voltageList


    #TDEVICE_API TED_BOOL TDeviceLdoAdcSetInConfig(TDEVICE_HDL hdl, int chIdx, int value);
    def LdoAdcSetInConfig(this, dutIdx, chIdx, value) :
        ret = this.__Api.TDeviceLdoAdcSetInConfig(this.__DeviceHandle, dutIdx, chIdx, value)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    #TDEVICE_API int TDeviceLdoAdcGetChannelCount(TDEVICE_HDL hdl);
    def LdoAdcGetChannelCount(this, dutIdx) :
        return this.__Api.TDeviceLdoAdcGetChannelCount(this.__DeviceHandle, dutIdx)

    #TDEVICE_API int TDeviceLdoAdcGetVoltage(TDEVICE_HDL hdl, int chIdx);
    def LdoAdcGetVoltage(this, dutIdx, chIdx) :
        return this.__Api.TDeviceLdoAdcGetVoltage(this.__DeviceHandle, dutIdx, chIdx)

    #TDEVICE_API TED_BOOL TDeviceLdoAdcGetAllVoltage(TDEVICE_HDL hdl, int* voltageArray);
    def LdoAdcGetAllVoltage(this, dutIdx) :

        structFmt = 'i'    #    TED_S32 no;    
        for i in range(15) :
            structFmt += 'i'    #    TED_S32 no;    
        structData = struct.pack(structFmt, 
                           0, 0, 0, 0, 0, 0, 0, 0,
                           0, 0, 0, 0, 0, 0, 0, 0,
                           )
        voltageList=[-1 for _ in range(16)]
        ret = this.__Api.TDeviceLdoAdcGetAllVoltage(this.__DeviceHandle, dutIdx, structData)
        if ret==1 :
            result= struct.unpack(structFmt, structData)
            for i in range(16) :
                voltageList[i] = result[i]
        return voltageList


    #TDEVICE_API TED_BOOL TDeviceRegAdcSetInConfig(TDEVICE_HDL hdl, int chIdx, int value);
    def RegAdcSetInConfig(this, dutIdx, chIdx, value) :
        ret = this.__Api.TDeviceRegAdcSetInConfig(this.__DeviceHandle, dutIdx, chIdx, value)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    #TDEVICE_API int TDeviceRegAdcGetChannelCount(TDEVICE_HDL hdl);
    def RegAdcGetChannelCount(this, dutIdx) :
        return this.__Api.TDeviceRegAdcGetChannelCount(this.__DeviceHandle, dutIdx)

    #TDEVICE_API int TDeviceRegAdcGetVoltage(TDEVICE_HDL hdl, int chIdx);
    def RegAdcGetVoltage(this, dutIdx, chIdx) :
        return this.__Api.TDeviceRegAdcGetVoltage(this.__DeviceHandle, dutIdx, chIdx)

    #TDEVICE_API TED_BOOL TDeviceLdoAdcGetAllVoltage(TDEVICE_HDL hdl, int* voltageArray);
    def RegAdcGetAllVoltage(this, dutIdx) :

        structFmt = 'i'    #    TED_S32 no;    
        for i in range(15) :
            structFmt += 'i'    #    TED_S32 no;    
        structData = struct.pack(structFmt, 
                           0, 0, 0, 0, 0, 0, 0, 0,
                           0, 0, 0, 0, 0, 0, 0, 0,
                           )
        voltageList=[-1 for _ in range(16)]
        ret = this.__Api.TDeviceRegAdcGetAllVoltage(this.__DeviceHandle, dutIdx, structData)
        if ret==1 :
            result= struct.unpack(structFmt, structData)
            for i in range(16) :
                voltageList[i] = result[i]
        return voltageList

    def AgingNotifyPyStart(this, pyFileName) : 
        ret = this.__Api.TDeviceAgingNotifyPyStart(this.__DeviceHandle, TString.ConvertToCTypeStrng(pyFileName))
        if ret==1 :
            bflag = True
        else :
            bflag = False
        return bflag

    def AgingNotifyPyStop(this, pyFileName) : 
        ret = this.__Api.TDeviceAgingNotifyPyStop(this.__DeviceHandle, TString.ConvertToCTypeStrng(pyFileName))
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    #TDEVICE_API TED_BOOL TDeviceAgingSetCurJobInfo(TDEVICE_HDL hdl, int jobID, int status, int scIdx, int scCount, const char* desc);
    def AgingSetCurJobInfo(this, dutIdx, jobID, status, scIdx, scCount, desc) :
        return this.__Api.TDeviceAgingSetCurJobInfo(this.__DeviceHandle, dutIdx, jobID, status,scIdx, scCount, TString.ConvertToCTypeStrng(desc))

    #TDEVICE_API TED_BOOL TDeviceAgingSetCurScInfo(TDEVICE_HDL hdl, int scID, int status, int tcIdx, int tcCount, const char* desc);
    def AgingSetCurScInfo(this, dutIdx, scID, status,tcIdx, tcCount, desc) :
        return this.__Api.TDeviceAgingSetCurScInfo(this.__DeviceHandle, dutIdx, scID, status,tcIdx, tcCount, TString.ConvertToCTypeStrng(desc))

    #TDEVICE_API TED_BOOL TDeviceAgingSetCurTcInfo(void* hdl, int dutIdx, int tcID, int status, int tcStepIdx, int tcStepCount, const char* desc);
    def AgingSetCurTcInfo(this, dutIdx, tcID, status, tcStepIdx, tcStepCount, desc) :
        return this.__Api.TDeviceAgingSetCurTcInfo(this.__DeviceHandle, dutIdx, tcID, status, tcStepIdx, tcStepCount, TString.ConvertToCTypeStrng(desc))

    #TDEVICE_API TED_BOOL TDeviceAgingSetCurTcStepInfo(TDEVICE_HDL hdl, int tcStepID, int status, const char* desc);
    def AgingSetCurTcStepInfo(this, dutIdx, tcStepID, status, desc) :
        return this.__Api.TDeviceAgingSetCurTcStepInfo(this.__DeviceHandle, dutIdx, tcStepID, status, TString.ConvertToCTypeStrng(desc))

    #TDEVICE_API  TED_BOOL TDeviceAgingMeasureADC(TDEVICE_HDL hdl, int jobID, int scID, int tcID, int tcStep,  const char* desc, /*OUT*/unsigned char* res);
    def AgingMeasureADC(this, dutIdx) : 
        #soutChannelCount = this.SoutAdcGetChannelCount()
        #ldoChannelCount = this.LdoAdcGetChannelCount()
        #regChannelCount = this.RegAdcGetChannelCount()
        #meaRes = Adc.Measure(soutChannelCount, ldoChannelCount, regChannelCount)
        ##this.__Api.TDeviceAgingMeasureADC(this.__DeviceHandle, dutIdx, meaRes.GetStructData())
        #meaRes.ParseStructData()
        #return meaRes
        this.__Api.TDeviceAgingMeasureADC(this.__DeviceHandle, dutIdx, None)
        #return True

    #COMM_API int TedAgingMeasureADCResultStructureByteSize();
    def AgingMeasureADCResultStructureByteSize(this) : 
        return this.__Api.TDeviceAgingMeasureADCResultStructureByteSize()
            
    def ANA670X_GetChipIDCount(this) : 
        return 1  #ted.ANA670X_GetChipIDCount()

    def ANA670X_GetChipID(this, dutIdx) : 
        return 1 #ted.ANA670X_GetChipID(dutIdx)

    def ANA670X_GetProductRevisionBytesCount(this) : 
        return 1 #ted.ANA670X_GetProductRevisionBytesCount()

    def ANA670X_GetProductRevisionBytes(this, dutIdx) : 
        return 1 #ted.ANA670X_GetProductRevisionBytes(dutIdx)

    def ANA670X_SetFrameRate(this, dutIdx, fr) :
        return True #ted.ANA670X_SetFrameRate(dutIdx, fr)

    def ANA670X_GetFrameRate(this, dutIdx) : 
        return 1 #ted.ANA670X_GetFrameRate(dutIdx)

    #COMM_API Bool TedAdcSoutSetRBSel(int dutIdx, int value)
    def AdcSoutSetRBSel(this, dutIdx, value) :
        return True #ted.AdcSoutSetRBSel(dutIdx, value)

    #COMM_API int TedAdcSoutGetRBSel(int dutIdx)
    def AdcSoutGetRBSel(this, dutIdx) :
        return 1 #ted.AdcSoutGetRBSel(dutIdx)


    #COMM_API Bool TedDD_DSIM_MipiReadReg(int dutIdx, int addr, int byteOffset, int readCount, unsigned char* buf, int bufMaxByteSize);
    def DD_DSIM_MipiReadReg(this, dutIdx, regAddr, byteOffset, readCount) : 
        return 1 #ted.DD_DSIM_MipiReadReg(dutIdx, regAddr, byteOffset, readCount)

    #COMM_API unsigned char TedDD_DSIM_MipiReadReg1Byte(int dutIdx, int addr, int byteOffset);
    def DD_DSIM_MipiReadReg1Byte(this, dutIdx, regAddr, byteOffset) : 
        return 1 #ted.DD_DSIM_MipiReadReg1Byte(dutIdx, regAddr, byteOffset)

    #COMM_API Bool TedDD_DSIM_MipiReadReg(int dutIdx, int addr, int byteOffset, int readCount, unsigned char* buf, int bufMaxByteSize);
    def DD_DSIM_MipiWriteReg(this, dutIdx, regAddr, byteOffset, regValueList) : 
        return 1 #ted.DD_DSIM_MipiWriteReg(dutIdx, regAddr, byteOffset, regValueList)

    #COMM_API Bool TedDD_DSIM_MipiWriteReg1Byte(int dutIdx, int addr, int byteOffset, unsigned char data);
    def DD_DSIM_MipiWriteReg1Byte(this, dutIdx, regAddr, byteOffset, regValue) : 
        return 1 #ted.DD_DSIM_MipiWriteReg1Byte(dutIdx, regAddr, byteOffset, regValue)

    #WREG0=0x39, [Addr], [regVal0], [regVal1].....
    def DD_DSIM_MipiWriteReg39(this, dutIdx, regAddr, regValueList) :
        #print("[this.__Api.TDevice.DD_DSIM_MipiWriteReg39] 0x%02x" % (regAddr))
        regValueByteArray=bytes(regValueList)
        ret = this.__Api.TDeviceDD_DSIM_MipiWriteReg39(this.__DeviceHandle, regAddr, len(regValueList), regValueByteArray)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    #WREG0=0x15, [Addr], [regVal]
    def DD_DSIM_MipiWriteReg15(this, dutIdx, regAddr, regValue) :

        #print("[this.__Api.TDevice.DD_DSIM_MipiWriteReg15] 0x%02x" % (regAddr))

        ret = this.__Api.TDeviceDD_DSIM_MipiWriteReg15(this.__DeviceHandle, regAddr, regValue)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    #WREG0=0x05, [Addr]
    def DD_DSIM_MipiWriteReg05(this, dutIdx, regAddr) :

        #print("[this.__Api.TDevice.DD_DSIM_MipiWriteReg05] 0x%02x" % (regAddr))

        ret = this.__Api.TDeviceDD_DSIM_MipiWriteReg05(this.__DeviceHandle, regAddr)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    #WREG0=0x07, [value]   :   Compressd Mode Command
    def DD_DSIM_MipiWriteReg07(this, dutIdx, value) :
        ret = this.__Api.TDeviceDD_DSIM_MipiWriteReg07(this.__DeviceHandle, value)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    
    
    def DD_DSIM_manual_ctrl(this, dutIdx, value) :
        ret = this.__Api.TDeviceDD_DSIM_manual_ctrl(this.__DeviceHandle, dutIdx, value)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag
        

    #COMM_API Bool TedDD_DSIM_power_ctrl(int dutIdx, int value);
    def DD_DSIM_power_ctrl(this, dutIdx, value) :
        ret = this.__Api.TDeviceDD_DSIM_power_ctrl(this.__DeviceHandle, dutIdx, value)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    #COMM_API Bool TedDD_DSIM_source_cal(int dutIdx, int value);
    def DD_DSIM_source_cal(this, dutIdx, value) :
        ret = this.__Api.TDeviceDD_DSIM_source_cal(this.__DeviceHandle, dutIdx, value)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    def DD_DSIM_sleepin(this, dutIdx, value) : 
        ret = this.__Api.TDeviceDD_DSIM_sleepin(this.__DeviceHandle, dutIdx, value)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag
    
    def DD_DSIM_sleepout(this, dutIdx, value) : 
        ret = this.__Api.TDeviceDD_DSIM_sleepout(this.__DeviceHandle, dutIdx, value)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag
    
    def DD_DSIM_deep_standby(this, dutIdx, value) : 
        ret = this.__Api.TDeviceDD_DSIM_deep_standby(this.__DeviceHandle, dutIdx, value)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag
    
    def DD_DSIM_displayon(this, dutIdx, value) : 
        ret = this.__Api.TDeviceDD_DSIM_displayon(this.__DeviceHandle, dutIdx, value)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag
    
    def DD_DSIM_reset_ctrl(this, dutIdx, value) : 
        ret = this.__Api.TDeviceDD_DSIM_reset_ctrl(this.__DeviceHandle, dutIdx, value)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag
    
    def DD_FB_blank(this, dutIdx, value) :
        ret = this.__Api.TDeviceDD_FB_blank(this.__DeviceHandle, dutIdx, value)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    def PatternConnect(this) : 
        ret = this.__Api.TDevicePatternConnect(this.__DeviceHandle)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    def PatternDisconnect(this) : 
        ret = this.__Api.TDevicePatternDisconnect(this.__DeviceHandle)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    def PatternIsConnect(this) : 
        ret = this.__Api.TDevicePatternIsConnect(this.__DeviceHandle)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag


    def PatternSetCommand(this, ptrnCmd) :
        ret = this.__Api.TDevicePatternSetCommand(this.__DeviceHandle, TString.ConvertToCTypeStrng(ptrnCmd))
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag
    
    def PatternPaint(this, r, g, b, a) : 
        ret = this.__Api.TDevicePatternPaint(this.__DeviceHandle, r, g, b, a)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    def PatternUpdateScreen(this) : 
        ret = this.__Api.TDevicePatternUpdateScreen(this.__DeviceHandle)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    def PatternDrawImage(this, imgFileName) : 
        ret = this.__Api.TDevicePatternDrawImage(this.__DeviceHandle, imgFileName)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    def PatternScreenVerify(this, r, g, b) : 
        ret = this.__Api.TDevicePatternScreenVerify(this.__DeviceHandle, r, g, b)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    
    def SysGetCurUtcTime(this) : 
        return this.__Api.TDeviceSysGetCurUtcTime(this.__DeviceHandle)

    def SysDelay(this, delay) :
        ret = this.__Api.TDeviceSysDelay(this.__DeviceHandle, delay)
        if ret==1 :
            bflag = True
        else :
            bflag = False;
        return bflag

    def SysGetDutIndexAllDeviceValue(this) : 
        return this.__Api.TDeviceSysGetDutIndexAllDeviceValue(this.__DeviceHandle)

    def SysGetDutCount(this) : 
        return this.__Api.TDeviceSysGetDutCount(this.__DeviceHandle)

    def SysGetTickCount64(this) : 
        return this.__Api.TDeviceSysGetTickCount64(this.__DeviceHandle)
    
    def SysGetUtcTimeKST(this, year, month, day, hour, min, sec) : 
        return this.__Api.TDeviceSysGetUtcTimeKST(this.__DeviceHandle, year, month, day, hour, min, sec)

    def SysGetErrFlag(this) : 
        return this.__Api.TDeviceSysGetErrFlag(this.__DeviceHandle)

    def SysMipiLock(this) : 
        return this.__Api.TDeviceSysMipiLock(this.__DeviceHandle)

    def SysMipiUnlock(this) : 
        return this.__Api.TDeviceSysMipiUnlock(this.__DeviceHandle)

    def SysMipiIsLock(this) : 
        return this.__Api.TDeviceSysMipiIsLock(this.__DeviceHandle)


#
# class TFileTransfer
#
class TFileTransfer :

    class Type(enum.IntEnum) : 
        T5 = 0
        
    class ErrorType(enum.IntEnum) : 
        Success = 0,
        SendPacket=1,
        NoResp=2,
        FileOpen=3,
        StorageSize=4,
        CRC=5
    
    #TDEVICE_API TFILETRANSFER_HDL TFileTransferCreate(enum TFileTransferType type, TDEVICE_HDL deviceHandle);
    def __init__(this, type, device) :
        this.__TFileTransferHandle = TFileTransferCreate(type, device.Handle)
        this.__FileName = ""

    def __getattr__(this, attrName) :
        if attrName == 'LastErrorString' : 
            return this.GetLastErrorString()
        if attrName == 'FileName' : 
            return this.__FileName
        else :
            raise AttributeError(attrName)


    #TDEVICE_API TED_BOOL TFileTransferDestroy(TFILETRANSFER_HDL fileTransferHandle);

    #TDEVICE_API TED_BOOL TFileTransferStart(TFILETRANSFER_HDL fileTransferHandle, const char* fileName);
    def Start(this, fileName) : 
        this.__FileName = fileName
        bytesString = fileName.encode('euc-kr')
        #bytesString = fileName.encode('ascii')
        #bytesString = fileName.encode('utf-8')
        ret = TFileTransferStart(this.__TFileTransferHandle, bytesString)
        return ret

    #TDEVICE_API TED_BOOL TFileTransferStop(TFILETRANSFER_HDL fileTransferHandle);
    def Stop(this) : 
        ret = TFileTransferStop(this.__TFileTransferHandle)
        return ret

    #TDEVICE_API int TFileTransferGetFileByteSize(TFILETRANSFER_HDL fileTransferHandle);
    def GetFileByteSize(this) : 
        ret = TFileTransferGetFileByteSize(this.__TFileTransferHandle)
        return ret

    #TDEVICE_API int TFileTransferGetTransferByteSize(TFILETRANSFER_HDL fileTransferHandle);
    def GetTransferByteSize(this) : 
        ret = TFileTransferGetTransferByteSize(this.__TFileTransferHandle)
        return ret

    #TDEVICE_API TED_BOOL TFileTransferIsStart(TFILETRANSFER_HDL fileTransferHandle);
    def IsStart(this) : 
        ret = TFileTransferIsStart(this.__TFileTransferHandle)
        return ret

    #TDEVICE_API TED_BOOL TFileTransferIsDone(TFILETRANSFER_HDL fileTransferHandle);
    def IsDone(this) : 
        ret = TFileTransferIsDone(this.__TFileTransferHandle)
        return ret

    #TDEVICE_API TED_BOOL TFileTransferIsError(TFILETRANSFER_HDL fileTransferHandle);
    def IsError(this) : 
        ret = TFileTransferIsError(this.__TFileTransferHandle)
        return ret

    #TDEVICE_API enum TFileTransferError TFileTransferGetLastError(TFILETRANSFER_HDL fileTransferHandle);
    def GetLastError(this) : 
        ret = TFileTransferGetLastError(this.__TFileTransferHandle)
        return ret

    def GetLastErrorString(this) :
        err = this.GetLastError()
        if err == TFileTransfer.ErrorType.Success :
            return "Success"
        elif err == TFileTransfer.ErrorType.SendPacket :
            return "SendPacket Error"
        elif err == TFileTransfer.ErrorType.NoResp :
            return "NoResp Error"
        elif err == TFileTransfer.ErrorType.FileOpen :
            return "FileOpen Error"
        elif err == TFileTransfer.ErrorType.StorageSize :
            return "StorageSize Error"
        elif err == TFileTransfer.ErrorType.CRC :
            return "CRC Error"
        else :
            return "Unknown Error"
    
    


from Anapass.TModule import *

class AppGlobal(object):

    def __init__(this):
        this.__ChipType = TChip.Type.Common
        this.__BoardType = TBoard.Type.Common
        this.__Device = TDevice(TDevice.Type.T5)
        isOk = this.__Device.Connect()
        if isOk != True :
            raise ConnectionRefusedError("Fail to connect to TDevice(%s)"%device.GetName())

    def GetDevice(this) :
        return this.__Device

    def GetChipType(this) :
        return this.__ChipType

    def SetChipType(this, chipType) :
        this.__ChipType = chipType


class ShellCommand :

    class ShellItem :
        def __init__(this, cmd, func, desc) :
            this.__Cmd = cmd
            this.__Func = func
            this.__Desc = desc

        def __repr__(this):
            str = this.__Cmd + "  " + this.__Desc
            return str

        def __str__(this) :
            str = this.__Cmd + "  " + this.__Desc
            return str

        def Run(this, app, cmd, strArg) :
            bRet = False
            str1 =cmd.lower()
            str2 =this.__Cmd.lower()
            if(str1 == str2) :
                bRet = True
                this.__Func(app, strArg)
            return bRet

    def __init__(this): 
        this.__Dictionary=dict()

    def Add(this, cmd, func, desc)  :
        item = ShellCommand.ShellItem(cmd, func, desc)
        this.__Dictionary[cmd] = item

    def PrintHelp(this) :
        for shellItem in this :
            print(shellItem)
   
    def __iter__(this) :
        this.__IterIndex = 0;
        return this

    def __next__(this) :
        if  this.__IterIndex >= len(this.__Dictionary) :
            raise StopIteration
        shellItem = list(this.__Dictionary.values())[this.__IterIndex];
        this.__IterIndex += 1;
        return shellItem








from Anapass.TModule import *
from Anapass.ANA6705 import ANA6705_FlashMipiToSpi
from Anapass.ANA6706 import ANA6706_FlashMipiToSpi
from abc import ABCMeta, abstractmethod
from Anapass import Util
import enum

class Type(enum.IntEnum) : 
        MipiToSpi=0

def CreateInstance(device, chipType, flashType, boardType=TBoard.Type.Common) :
        if chipType == TChip.Type.ANA6705 :
            flash = FlashMipiToSpiANA6705(device)
        elif chipType == TChip.Type.ANA6706 :
            flash = FlashMipiToSpiANA6706(device)
        else : 
            raise ArgumentError("[Flash::CreateInstance] The value of ChipType is invalid")
        return flash


########################################################################################################################
#
#  class Flash
#
########################################################################################################################
class Flash(metaclass=ABCMeta) :
    def __init__(this, device):
        this._Device = device

    @abstractmethod
    def GetCapacityByteSize(this) : pass
    @abstractmethod
    def GetSectorMaxCount(this) : pass
    @abstractmethod
    def ConvertMemAddrToSectorAddr(this, memAddr) : pass
    @abstractmethod
    def ConvertSectorAddrToMemAddr(this, sectorAddr) : pass
    @abstractmethod
    def GetFlashReadChunkByteSize(this) : pass
    @abstractmethod
    def GetFlashWriteChunkByteSize(this) : pass
    @abstractmethod
    def GetFlashSectorByteSize(this) : pass

    @abstractmethod
    def ReadStatus1(this) : pass
    @abstractmethod
    def ReadStatus2(this) : pass
    @abstractmethod
    def WriteStatus1(this, status) : pass
    @abstractmethod
    def WriteStatus2(this, status) : pass
    @abstractmethod
    def Lock(this) : pass
    @abstractmethod
    def Unlock(this) : pass
    @abstractmethod
    def IsLock(this) : pass
    @abstractmethod
    def EraseSector(this, sectorAddr) : pass
    @abstractmethod
    def ReadDataFromHW(this, memAddr, readByteSize, dataList, dataListStartIdx) : pass
    @abstractmethod
    def WriteDataToHW(this, memmAddr, writeByteSize, writeData, dataIdx) : pass

    #API
    def Read(this, memAddr, readDataByteSize, outData, workAgent=None) : 
        bRet = False
        flashChunkByteSize = this.GetFlashReadChunkByteSize()
        flashSectorByteSize = this.GetFlashSectorByteSize()
        curDataIdx = 0;
        curMemAddr = memAddr; 

        while curDataIdx < readDataByteSize :
            if ((curDataIdx + flashChunkByteSize) > readDataByteSize) :
                readByteSize = readDataByteSize - curDataIdx;
            else :
                readByteSize = flashChunkByteSize;

            bRet = this.ReadDataFromHW(curMemAddr, readByteSize, outData, curDataIdx);
            if bRet != True :
                break

            curDataIdx += readByteSize;
            curMemAddr += readByteSize;

            if workAgent != None :
                workAgent.IncCurValue(readByteSize)
       
        return bRet;

    #API
    def WriteToSector(this, sectorAddr, writeData, writeDataStartIdx, workAgent=None) : 

        bRet = False
        flashChunkByteSize = this.GetFlashWriteChunkByteSize();
        flashSectorByteSize = this.GetFlashSectorByteSize();
        curDataIdx = 0;
        curMemAddr = this.ConvertSectorAddrToMemAddr(sectorAddr);

        while (curDataIdx < flashSectorByteSize) :

            if ((curDataIdx + flashChunkByteSize) > flashSectorByteSize) :
                writeByteSize = flashSectorByteSize - curDataIdx;
            else :
                writeByteSize = flashChunkByteSize;
            

            bRet = this.WriteDataToHW(curMemAddr, writeByteSize, writeData, curDataIdx + writeDataStartIdx)
            assert bRet

            curDataIdx += writeByteSize;
            curMemAddr += writeByteSize;

            if workAgent != None :
                workAgent.IncCurValue(writeByteSize)
        
        return bRet;

    #API  //임의 크기의 데이타를  임의의 메모리에 쓴다.
    def WriteToMemory(this, memAddr, writeBytes, writenData, writeDataStartIdx, workAgent=None) : 
        bRet = False
        flashChunkByteSize = this.GetFlashWriteChunkByteSize();
        curDataIdx = 0
        curMemAddr = memAddr

        assert ((memAddr + writeDataSize) < this.GetCapacityByteSize())

        while (curDataIdx < writeDataSize) :

            if ((curDataIdx + flashChunkByteSize) > writeDataSize) :
                writeByteSize = writeDataSize - curDataIdx;
            else :
                writeByteSize = flashChunkByteSize;
            
            bRet = this.WriteDataToHW(curMemAddr, writeByteSize, writeData, curDataIdx+writeDataStartIdx)
            assert bRet

            curDataIdx += writeByteSize;
            curMemAddr += writeByteSize;

            #if (curWrittenByteSize) *curWrittenByteSize += writeByteSize;
            #if (isBreak) {
            #    if (*isBreak) {
            #        break;
            #    }
            #}

            if workAgent != None :
                workAgent.IncCurValue(writeByteSize)

        
        return bRet;

    #API
    def Write(this, memAddr, writeDataByteSize, writeData, workAgent=None) : 

        bRet = False
        dataIdx = 0
        curWriteMemAddr = memAddr;
        sectorByteSize = this.GetFlashSectorByteSize();

        assert  (memAddr + writeDataByteSize) <= this.GetCapacityByteSize()

        while (dataIdx < writeDataByteSize) :

            writeBytes = 0
            availWriteBytes = 0
            curWriteSectorAddr = this.ConvertMemAddrToSectorAddr(curWriteMemAddr);
            curWriteSectorStartMemAddr = this.ConvertSectorAddrToMemAddr(curWriteSectorAddr);
            
            #현재 Sector에서 Write가능한 ByteSize를 구한다. 
            if (curWriteSectorStartMemAddr != curWriteMemAddr) :
                assert  (curWriteMemAddr > curWriteSectorStartMemAddr)
                availWriteBytes = sectorByteSize - (curWriteMemAddr - curWriteSectorStartMemAddr);
            else :
                availWriteBytes = sectorByteSize;

            #실제 Write할 ByteSize를 구한다. 
            if ((dataIdx + availWriteBytes) < writeDataByteSize) :
                writeBytes = availWriteBytes;
            else :
                writeBytes = writeDataByteSize - dataIdx;

            #writeBytes가 Sector전체 크기와 같다면, Erase한다음  WriteToSector를 호출한다 
            if (writeBytes == sectorByteSize) :

                bRet = this.EraseSector(curWriteSectorAddr);
                assert bRet

                bRet = this.WriteToSector(curWriteSectorAddr, writeData, dataIdx, workAgent)
                assert bRet
            
            #writeBytes가 Sector보다 작다면, Sector를 읽고, Erase한다음 , Write한다.
            elif (writeBytes < sectorByteSize) :
            
                assert (curWriteMemAddr >= curWriteSectorStartMemAddr)

                #Read시간이 너무 오래 걸린다.  Sector단위로 Write한다. 
                tmpData = [0 for _ in range(sectorByteSize)] 
                secotrMemOffset = curWriteMemAddr - curWriteSectorStartMemAddr  # Sector시작 주소로부터의 메모리 Offset

                Util.ListCopy(tmpData, secotrMemOffset, writeData, dataIdx, writeBytes)

                #Sector Erase한다. 
                bRet = this.EraseSector(curWriteSectorAddr)
                assert bRet

                #Data를 쓴다. 
                bRet = this.WriteToSector(curWriteSectorAddr, tmpData, 0, workAgent);
                assert bRet


                # 1Sector를 읽은다음, 값변경후 Write하기 때문에  WriteToSector에 curWrittenByteSize를 넣으면  실제 쓴값보다 큰 값이 나온다.
                #if (curWrittenByteSize) *curWrittenByteSize += writeBytes;
            else :
                bRet = this.WriteToMemory(curWriteMemAddr, writeBytes, writenData, dataIdx, workAgent);
                assert bRet
            
            
            curWriteMemAddr += writeBytes;
            dataIdx += writeBytes;

        return bRet;



class FlashMipiToSpiANA6705(Flash) :
    def __init__(this, device):
        Flash.__init__(this, device)

    def GetCapacityByteSize(this) : 
        return ANA6705_FlashMipiToSpi.FlashCapacity

    def GetSectorMaxCount(this) : 
        return ANA6705_FlashMipiToSpi.FlashSectorMaxCount
        
    def GetFlashReadChunkByteSize(this) : 
        return ANA6705_FlashMipiToSpi.FlashReadChunkByteSize

    def GetFlashWriteChunkByteSize(this) : 
        return ANA6705_FlashMipiToSpi.FlashWriteChunkByteSize

    def GetFlashSectorByteSize(this) : 
        return ANA6705_FlashMipiToSpi.FlashSectorByteSize

    def ConvertMemAddrToSectorAddr(this, memAddr) : 
        return ANA6705_FlashMipiToSpi.ConvertMemAddrToSectorAddr(memAddr)

    def ConvertSectorAddrToMemAddr(this, sectorAddr) : 
        return ANA6705_FlashMipiToSpi.ConvertSectorAddrToMemAddr(sectorAddr)

    def ReadStatus1(this) : 
        return ANA6705_FlashMipiToSpi.ReadStatus1(this._Device)

    def ReadStatus2(this) : 
        return ANA6705_FlashMipiToSpi.ReadStatus2(this._Device)

    def WriteStatus1(this, status) : 
        return ANA6705_FlashMipiToSpi.WriteStatus1(this._Device, status)

    def WriteStatus2(this, status) : 
        return ANA6705_FlashMipiToSpi.WriteStatus2(this._Device, status)

    def Lock(this) : 
        ANA6705_FlashMipiToSpi.Lock(this._Device)

    def Unlock(this) : 
        ANA6705_FlashMipiToSpi.Unlock(this._Device)

    def IsLock(this) : 
        return ANA6705_FlashMipiToSpi.IsLock(this._Device)

    def EraseSector(this, sectorAddr) : 
        return ANA6705_FlashMipiToSpi.EraseSector(this._Device, sectorAddr)

    def ReadDataFromHW(this, memAddr, readByteSize, dataList, dataListStartIdx) : 
        return ANA6705_FlashMipiToSpi.ReadDataFromHW(this._Device, memAddr, readByteSize, dataList, dataListStartIdx)

    def WriteDataToHW(this, memAddr, writeByteSize, writeData, dataIdx) : 
        return ANA6705_FlashMipiToSpi.WriteDataToHW(this._Device, memAddr, writeByteSize, writeData, dataIdx)

class FlashMipiToSpiANA6706(Flash) :
    def __init__(this, device):
        Flash.__init__(this, device)

    def GetCapacityByteSize(this) : 
        return ANA6706_FlashMipiToSpi.FlashCapacity

    def GetSectorMaxCount(this) : 
        return ANA6706_FlashMipiToSpi.FlashSectorMaxCount
        
    def GetFlashReadChunkByteSize(this) : 
        return ANA6706_FlashMipiToSpi.FlashReadChunkByteSize

    def GetFlashWriteChunkByteSize(this) : 
        return ANA6706_FlashMipiToSpi.FlashWriteChunkByteSize

    def GetFlashSectorByteSize(this) : 
        return ANA6706_FlashMipiToSpi.FlashSectorByteSize

    def ConvertMemAddrToSectorAddr(this, memAddr) : 
        return ANA6706_FlashMipiToSpi.ConvertMemAddrToSectorAddr(memAddr)

    def ConvertSectorAddrToMemAddr(this, sectorAddr) : 
        return ANA6706_FlashMipiToSpi.ConvertSectorAddrToMemAddr(sectorAddr)

    def ReadStatus1(this) : 
        return ANA6706_FlashMipiToSpi.ReadStatus1(this._Device)

    def ReadStatus2(this) : 
        return ANA6706_FlashMipiToSpi.ReadStatus2(this._Device)

    def WriteStatus1(this, status) : 
        return ANA6706_FlashMipiToSpi.WriteStatus1(this._Device, status)

    def WriteStatus2(this, status) : 
        return ANA6706_FlashMipiToSpi.WriteStatus2(this._Device, status)

    def Lock(this) : 
        ANA6706_FlashMipiToSpi.Lock(this._Device)

    def Unlock(this) : 
        ANA6706_FlashMipiToSpi.Unlock(this._Device)

    def IsLock(this) : 
        return ANA6706_FlashMipiToSpi.IsLock(this._Device)

    def EraseSector(this, sectorAddr) : 
        return ANA6706_FlashMipiToSpi.EraseSector(this._Device, sectorAddr)

    def ReadDataFromHW(this, memAddr, readByteSize, dataList, dataListStartIdx) : 
        return ANA6706_FlashMipiToSpi.ReadDataFromHW(this._Device, memAddr, readByteSize, dataList, dataListStartIdx)

    def WriteDataToHW(this, memAddr, writeByteSize, writeData, dataIdx) : 
        return ANA6706_FlashMipiToSpi.WriteDataToHW(this._Device, memAddr, writeByteSize, writeData, dataIdx)






from Anapass.TModule import *
from Anapass import Aging
import inspect

def TEDMode_HLPM_30Hz_ON(device, dutIdx, isDelay=True) :
    
    print()
    print("[TEDMode_HLPM_30Hz_ON] Enter ")

    HZ = 30
    frameRate = device.ANA670X_GetFrameRate(dutIdx)
    if frameRate != HZ :

        #VEH, VAINT 측정을 위하여 PWR_MODE 설정을 1로 변경 필요,  6A72h → 0x48로 설정 변경
        device.WREG0_15(dutIdx, 0xB0, 0x72) 
        device.WREG0_15(dutIdx, 0x6A, 0x48)  
    
        while True :
            device.ANA670X_SetFrameRate(dutIdx, HZ) #device.WREG0_15(dutIdx, 0x53, 0x24)  #HLPM ON
            frameRate = device.ANA670X_GetFrameRate(dutIdx)
            print("[TEDMode_HLPM_30Hz_ON] CurHz=%d" % frameRate)
            if frameRate == HZ :
                break

        print("[TEDMode_HLPM_30Hz_ON] Leave Hz=%d" % frameRate)
    else :
        print("[TEDMode_HLPM_30Hz_ON] Skip Hz=%d" % frameRate)

    print()
    

def TEDMode_HS1_120Hz(device, dutIdx) :
    
    print()
    print("[TEDMode_HS1_120Hz] Enter ")

    HZ = 120
    frameRate = device.ANA670X_GetFrameRate(dutIdx)
    if frameRate != HZ :

        #Normal모드에서 VBOT값 흔들림 방지를 위해 6A72h → 0x40로 설정 변경
        device.WREG0_15(dutIdx, 0xB0, 0x72) 
        device.WREG0_15(dutIdx, 0x6A, 0x40)  

        while True : 
            #TEDMode_HLPM_30Hz_OFF(device, dutIdx, False)
            device.ANA670X_SetFrameRate(dutIdx, HZ) 
            #device.WREG0_15(dutIdx, 0xB0, 0x01)  #echo "0x15 0xB0 0x01" > /sys/devices/platform/12870000.dsim/mipi2_rw
            #device.WREG0_15(dutIdx, 0x60, 0x00)  #echo "0x39 0x60 0x00" > /sys/devices/platform/12870000.dsim/mipi2_rw
            #device.WREG0_15(dutIdx, 0xF7, 0x07)  #Update
                                             #echo "0x15 0xB0 0x00" > /sys/devices/platform/12870000.dsim/mipi2_rw
                                             #echo "0x39 0xF7 0x07" > /sys/devices/platform/12870000.dsim/mipi2_rw
                                             # Check Mode
                                             # cat /sys/devices/platform/12860000.decon_f/tetime
                                             #    TE_TIME: 8278895
            
            frameRate = device.ANA670X_GetFrameRate(dutIdx)
            print("[TEDMode_HS1_120Hz] Cur Hz=%d" % frameRate)
            if frameRate == HZ :
                break;

        print("[TEDMode_HS1_120Hz] Setting Leave Hz=%d" % frameRate)

    else :
        print("[TEDMode_HS1_120Hz] Setting Skip Hz=%d" % frameRate)

    print()


def TEDMode_Normal_60Hz(device, dutIdx) :
    
    print()
    #print("[TEDMode_Normal_60Hz] Enter TEDMode_Delay=%d" % TEDMode_Delay)
    print("[TEDMode_Normal_60Hz] Enter ")

    HZ = 50
    frameRate = device.ANA670X_GetFrameRate(dutIdx)
    if frameRate != 60 :
        #Normal모드에서 VBOT값 흔들림 방지를 위해 6A72h → 0x40로 설정 변경
        device.WREG0_15(dutIdx, 0xB0, 0x72) 
        device.WREG0_15(dutIdx, 0x6A, 0x40)  

        while True : 
            device.ANA670X_SetFrameRate(dutIdx, 60) 
            frameRate = device.ANA670X_GetFrameRate(dutIdx)
            print("[TEDMode_Normal_60Hz] Cur Hz=%d" % frameRate)
            if frameRate == 60 :
                break
        print("[TEDMode_Normal_60Hz] Leave Hz=%d" % frameRate)
    else :
        print("[TEDMode_Normal_60Hz] Skip Hz=%d" % frameRate)
    print()


def TCBegin(device, dutIdx) :

    if platform.system()=="Windows" : 
        delayTick=0
    else :
        delayTick=2000

    #Blank On/Off
    device.DD_FB_blank(dutIdx, 1)         #echo 1 > /sys/class/graphics/fb0/blank;
    device.SysDelay(delayTick)
    device.DD_FB_blank(dutIdx, 0)         #echo 0 > /sys/class/graphics/fb0/blank;
    device.SysDelay(delayTick)
        
    # Blank Off 시 , 120Hz, source cal 하므로 설정할 필요없다. 
    #120Hz
    #TEDMode_HS1_120Hz(device, dutIdx)

    #Source Cal
    #device.DD_DSIM_source_cal(dutIdx, 1) #echo 1 > /sys/devices/platform/12870000.dsim/source_cal

    #Write Black
    device.PatternConnect()
    device.PatternPaint(0, 0, 0, 0);
    device.PatternUpdateScreen()
    device.SysDelay(500)

    # Screen Verify
    device.PatternScreenVerify(0, 0, 0)


def TCEnd(device, dutIdx) :

    if platform.system()=="Windows" : 
        delayTick=0
    else :
        delayTick=2000

    #Write Black
    device.PatternDisconnect()
    
    #120Hz  # Blank Off 시 , 120Hz, source cal 하므로 설정할 필요없다. 
    #TEDMode_HS1_120Hz(device, dutIdx)

    #Blank On/Off
    device.DD_FB_blank(dutIdx, 1)         #echo 1 > /sys/class/graphics/fb0/blank;
    device.SysDelay(delayTick)
    device.DD_FB_blank(dutIdx, 0)         #echo 0 > /sys/class/graphics/fb0/blank;
    device.SysDelay(delayTick)
        
    
def VEH_BML_Setting(device, dutIdx) :
    #VEH, BML 측정을 위하여 PWR_MODE 설정을 1로 변경 필요,  6A72h → 0x48로 설정 변경
    device.WREG0_15(dutIdx, 0xB0, 0x72) 
    device.WREG0_15(dutIdx, 0x6A, 0x48)  
    
    #VGLL, VGHH, VAINT, VEH, BML 측정을 위한 LDO Enable, F43Dh, F43Eh → 0x00으로 설정 변경
    device.WREG0_15(dutIdx, 0xB0, 0x3D) 
    device.WREG0_15(dutIdx, 0xF4, 0x00)  

    device.WREG0_15(dutIdx, 0xB0, 0x3E) 
    device.WREG0_15(dutIdx, 0xF4, 0x00)  
